package com.bocloud.sms.booter.config;

import com.bocloud.common.model.RequestContext;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.repository.TenantRepository;
import com.bocloud.sms.repository.UserRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * @author tyl
 * @since 2020/3/25 周三 14:06
 */
@Component
public class AccountCache implements InitializingBean {


    private final StringRedisTemplate redisTemplate;
    private final TenantRepository tenantRepository;
    private final UserRepository userRepository;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final Logger logger = LoggerFactory.getLogger(AccountCache.class);

    public AccountCache(StringRedisTemplate redisTemplate, TenantRepository tenantRepository, UserRepository userRepository, ThreadPoolTaskExecutor taskExecutor) {
        this.redisTemplate = redisTemplate;
        this.tenantRepository = tenantRepository;
        this.userRepository = userRepository;
        this.taskExecutor = taskExecutor;
    }

    public void putAccountCache(RequestContext.Catalog catalog, Long id, String account) {
        String key = catalog.name() + "_" + id;
        this.redisTemplate.opsForValue().set(key, account);
    }

    public void putNameCache(RequestContext.Catalog catalog, Long id, String name) {
        String key = catalog.name() + "_Name_" + id;
        this.redisTemplate.opsForValue().set(key, name);
    }

    /**
     * 获取缓存中用户账号
     *
     * @param id 用户id
     * @return
     */
    public String getUserAccount(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_" + id);
    }

    /**
     * 获取缓存中用户名称
     *
     * @param id 用户id
     * @return
     */
    public String getUserName(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + "_Name_" + id);
    }

    /**
     * 获取缓存中租户账号
     *
     * @param id 租户id
     * @return
     */
    public String getTenantAccount(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + "_" + id);
    }

    /**
     * 获取缓存中租户名称
     *
     * @param id 租户id
     * @return
     */
    public String getTenantName(Long id) {
        return redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + "_Name_" + id);
    }

    @Override
    public void afterPropertiesSet() {
        taskExecutor.execute(this::addTenantCache);
        taskExecutor.execute(this::addUserCache);
    }


    public void addTenantCache() {
        try {
            List<Tenant> tenants = tenantRepository.list();
            //缓存租户账号和租户名
            for (Tenant tenant : tenants) {
                Long id = tenant.getId();
                if (!StringUtils.hasText(getTenantAccount(id))) {
                    putAccountCache(RequestContext.Catalog.Tenant, tenant.getId(), tenant.getAccount());
                }
                if (!StringUtils.hasText(getTenantName(id))) {
                    putNameCache(RequestContext.Catalog.Tenant, tenant.getId(), tenant.getAccount());
                }
            }
        } catch (Exception e) {
            logger.error("put tenant cache error,", e);
        }
    }

    public void addUserCache() {
        try {
            List<User> users = userRepository.list();
            User admin = userRepository.query(1L);
            users.add(admin);
            //缓存用户账号和用户名
            for (User user : users) {
                Long id = user.getId();
                if (!StringUtils.hasText(getUserAccount(id))) {
                    putAccountCache(RequestContext.Catalog.User, user.getId(), user.getAccount());
                }
                if (!StringUtils.hasText(getUserName(id))) {
                    putNameCache(RequestContext.Catalog.User, user.getId(), user.getAccount());
                }
            }
        } catch (Exception e) {
            logger.error("put user cache error,", e);
        }
    }
}
