package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Business;
import com.bocloud.sms.entity.Project;
import com.bocloud.sms.interfaces.BusinessService;
import com.bocloud.sms.model.BusinessModel;
import com.bocloud.sms.model.ImpExpBusinessExcelModel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author tyl
 * @time 2020/6/3 周三 15:29
 */
@RestController @RequestMapping("/v1/bizs") @Api(value = "业务管理", tags = "业务管理") public class BusinessController {

    private final Logger logger = LoggerFactory.getLogger(BusinessController.class);
    private final BusinessService businessService;

    public BusinessController(BusinessService businessService) {
        this.businessService = businessService;
    }

    /**
     * 查看业务列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查看业务列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping public BocloudResult<GridBean<Business>> list(Pager pager,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.list(pager, requestContext);
    }

    /**
     * 添加业务
     *
     * @param businessModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "添加业务", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping public Result create(@RequestBody BusinessModel businessModel,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.create(businessModel, requestContext);
    }

    /**
     * 修改业务
     *
     * @param id
     * @param businessModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "修改业务", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}") public Result modify(@PathVariable(value = Common.ID) Long id,
        @RequestBody BusinessModel businessModel, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.modify(id, businessModel, requestContext);
    }

    /**
     * 删除业务
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "删除业务", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}") public Result remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.remove(id, requestContext);
    }

    /**
     * 批量删除业务
     *
     * @param ids
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {
        "CMC"}, value = "批量删除业务", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping public Result batchRemove(@RequestBody List<Long> ids,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return businessService.batchRemove(ids, requestContext);
    }

    /**
     * 查看业务明细
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查看业务明细", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}") public BocloudResult<Business> detail(@PathVariable(value = Common.ID) Long id) {
        return businessService.detail(id);
    }

    /**
     * 查看业务下项目
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "查看业务下项目", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/projects") public BocloudResult<List<Project>> projects(
        @PathVariable(value = Common.ID) Long id) {
        return businessService.listBusinessProject(id);
    }

    /**
     * 配置业务项目
     *
     * @param id
     * @param projectIds
     * @return
     */
    @ApiOperation(tags = {"CMC",
        "CSC"}, value = "配置业务项目", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}") public Result businessAction(@PathVariable(value = Common.ID) Long id,
        @RequestBody List<Long> projectIds) {
        return businessService.businessAction(id, projectIds);
    }

    /**
     * 导出业务列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated @ApiOperation(tags = {"CMC",
        "CSC"}, value = "导出项目列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/export", method = {RequestMethod.GET}) public void export(Pager pager,
        @Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request,
        HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        try {
            XSSFWorkbook workbook = businessService.export(paramList, requestContext);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "业务统计数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            logger.error("export project error,", e);
        }
    }

    @ApiOperation(value = "下载业务列表模板", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletResponse response) {
         businessService.importTemp(response);
    }

    @ApiOperation(value = "导入业务列表", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest)request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return businessService.importData(multipartFile, requestContext);
    }

}

