package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.DepartmentService;
import com.bocloud.sms.model.DepartmentModel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 组织管理控制器
 *
 * @author tyl
 * @since 2021/03/25
 */
@RestController
@RequestMapping("/v1/departments")
@Api(value = "组织管理", tags = "组织管理")
public class DepartmentController {

    private final DepartmentService departmentService;

    public DepartmentController(DepartmentService departmentService) {
        this.departmentService = departmentService;
    }

    /**
     * 查询组织机构
     *
     * @param parentId
     * @param requestContext
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CSC"}, value = "组织机构列表显示", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<Department>> list(@RequestParam(value = Common.PARENTID) Long parentId, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.list(parentId, requestContext);
    }

    /**
     * 根据层级查询组织机构
     *
     * @param parentId
     * @param requestContext
     * @return
     */
    @GetMapping("/listByParentId")
    @ApiOperation(tags = {"CSC"}, value = "更具层级查询组织机构", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<Department>> listByParentId(@RequestParam(value = Common.PARENTID) Long parentId, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.listByParentId(parentId, requestContext);
    }

    /**
     * 查询组织机构下的管理员
     *
     * @param id
     * @param requestContext
     * @return
     */
    @GetMapping("/{id}/managers")
    @ApiOperation(tags = {"CSC"}, value = "查询组织机构下的管理员", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<User>> listDeptManager(@PathVariable(value = Common.ID) Long id, RequestContext requestContext) {
        return departmentService.listDeptManager(id, requestContext);
    }

    /**
     * 创建组织机构
     *
     * @param department
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "创建组织机构", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody DepartmentModel department,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.create(department, requestContext);
    }

    /**
     * 修改组织机构
     *
     * @param id
     * @param department
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "修改组织机构", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody DepartmentModel department,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.modify(id, department, requestContext);
    }

    /**
     * 删除组织机构
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "删除组织机构", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return departmentService.remove(id, requestContext);
    }

    /**
     * 查询组织机构详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "组织机构详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Department> detail(@PathVariable(value = Common.ID) Long id) {
        return departmentService.detail(id);
    }

}
