package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.service.utils.IpTool;
import com.bocloud.sms.entity.AccountSecurity;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.entity.TenantAccount;
import com.bocloud.sms.interfaces.ManagerService;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.interfaces.UserService;
import com.bocloud.sms.model.LoginModel;
import com.bocloud.sms.repository.SystemConfigRepository;
import com.bocloud.sms.service.internel.CosInternelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 登录控制器
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1")
@Api(value = "登录管理", tags = "登录管理")
public class LoginController {

    private final UserService userService;
    private final TenantService tenantService;
    private final ManagerService managerService;
    private final CosInternelService cosInternelService;
    private final SystemConfigRepository systemConfigRepository;
    private final StringRedisTemplate redisTemplate;
    private final LockFactory lockFactory;
    private final Logger logger = LoggerFactory.getLogger(LoginController.class);

    public LoginController(UserService userService, TenantService tenantService, ManagerService managerService,
                           CosInternelService cosInternelService, SystemConfigRepository systemConfigRepository,
                           StringRedisTemplate redisTemplate, LockFactory lockFactory) {
        this.userService = userService;
        this.tenantService = tenantService;
        this.managerService = managerService;
        this.cosInternelService = cosInternelService;
        this.systemConfigRepository = systemConfigRepository;
        this.redisTemplate = redisTemplate;
        this.lockFactory = lockFactory;
    }

    /**
     * 用户登录
     *
     * @param loginModel
     * @param request
     * @return
     */
    @ApiOperation(tags = {
            "CSC"}, value = "用户登录", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/users/login")
    public BocloudResult<Map<String, Object>> userLogin(@RequestBody LoginModel loginModel,
                                                        HttpServletRequest request) {
        String account = loginModel.getAccount();
        String password = loginModel.getPassword();
        BocloudResult<Map<String, Object>> loginResult = userService
                .login(account, password, request.getSession().getId(), IpTool.getIP(request), loginModel.getIsManager());
        if (loginResult.isFailed()) {
            if (null == loginResult.getData()) {
                String path = "Login_" + account;
                try (BocloudACLock lock = lockFactory.getACLock(path)) {
                    Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
                    String value = redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + account);
                    SystemConfig errorNumber = systemConfigRepository.queryByCode("pwdErrorNumber");
                    int error = Integer.parseInt(errorNumber.getValue());
                    if (!StringUtils.hasText(value)) {
                        //密码操作失败
                        redisTemplate.opsForValue()
                                .set(RequestContext.Catalog.User.name() + account, String.valueOf(1), 60, TimeUnit.SECONDS);
                        loginResult = new BocloudResult<>(false, loginResult.getMessage() + ",已操作【1】次");
                    } else if (Integer.parseInt(value) >= error) {
                        Result lockResult = userService.lockByAccount(account);
                        boolean success = lockResult.isSuccess();
                        redisTemplate.delete(RequestContext.Catalog.User.name() + account);
                        if (success) {
                            loginResult = new BocloudResult<>(false, "账号或者密码错误已达【" + error + "】次，若用户存在将被冻结，请联系管理员解冻");
                        } else {
                            loginResult = new BocloudResult<>(false, "认证服务账号缓存存在问题");
                        }
                    } else {
                        redisTemplate.boundValueOps(RequestContext.Catalog.User.name() + account).increment(1L);
                        String num = redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + account);
                        loginResult = new BocloudResult<>(false, loginResult.getMessage() + ",已操作【" + num + "】次");
                    }
                    return loginResult;
                } catch (Exception e) {
                    logger.error("Get login error message：", e);
                    return loginResult;
                }
            }
        }
        redisTemplate.delete(RequestContext.Catalog.User.name() + account);
        return loginResult;
    }

    /**
     * 用户登出
     *
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {
            "CSC"}, value = "用户退出系统", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/users/logout")
    public Result userLogout(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.logout(requestContext.getTarget());
    }

    /**
     * 锁定用户
     *
     * @param account
     * @return
     */
    @ApiOperation(tags = {
            "CSC"}, value = "锁定用户", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/users")
    @Deprecated
    public Result userLock(@RequestParam(value = Common.ACCOUNT) String account) {
        return userService.lockByAccount(account);
    }

    /**
     * 租户登录
     *
     * @param loginModel
     * @param request
     * @return
     */
    @ApiOperation(tags = {
            "CSC"}, value = "租户登录", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/tenants/login")
    public BocloudResult<Map<String, Object>> tenantLogin(
            @RequestBody LoginModel loginModel, HttpServletRequest request) {
        String account = loginModel.getAccount();
        String password = loginModel.getPassword();
        BocloudResult<Map<String, Object>> loginResult =
                tenantService.login(account, password, request.getSession().getId(), IpTool.getIP(request));
        if (loginResult.isFailed()) {
            if (null == loginResult.getData()) {
                String path = "Login_" + account;
                try (BocloudACLock lock = lockFactory.getACLock(path)) {
                    Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
                    String value = redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + account);
                    SystemConfig errorNumber = systemConfigRepository.queryByCode("pwdErrorNumber");
                    int error = Integer.parseInt(errorNumber.getValue());
                    if (!StringUtils.hasText(value)) {
                        //密码操作失败
                        redisTemplate.opsForValue()
                                .set(RequestContext.Catalog.Tenant.name() + account, String.valueOf(1), 60,
                                        TimeUnit.SECONDS);
                        loginResult = new BocloudResult<>(false, loginResult.getMessage() + ",已操作【1】次");
                    } else if (Integer.parseInt(value) >= error) {
                        Result lockResult = tenantService.lockByAccount(account);
                        boolean success = lockResult.isSuccess();
                        redisTemplate.delete(RequestContext.Catalog.Tenant.name() + account);
                        if (success) {
                            loginResult = new BocloudResult<>(false, "账号或者密码错误已达【" + error + "】次，若租户存在将被冻结，请联系管理员解冻");
                        } else {
                            loginResult = new BocloudResult<>(false, "认证服务存在问题");
                        }
                    } else {
                        redisTemplate.boundValueOps(RequestContext.Catalog.Tenant.name() + account).increment(1L);
                        String num = redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + account);
                        loginResult = new BocloudResult<>(false, loginResult.getMessage() + ",已操作【" + num + "】次");
                    }
                    return loginResult;
                } catch (Exception e) {
                    logger.error("Get error message：", e);
                    return loginResult;
                }
            }
        }
        redisTemplate.delete(RequestContext.Catalog.Tenant.name() + account);
        return loginResult;
    }

    /**
     * 租户登出
     *
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {
            "CSC"}, value = "租户登出", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/tenants/logout")
    public Result tenantLogout(
            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.logout(requestContext.getTarget());
    }

    /**
     * 锁定租户
     *
     * @param account
     * @return
     */
    @ApiOperation(tags = {
            "CSC"}, value = "锁定租户", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/tenants")
    public Result tenantLock(@RequestParam(value = Common.ACCOUNT) String account) {
        return tenantService.lockByAccount(account);
    }

    /**
     * 管理员登录
     *
     * @param loginModel
     * @param request
     * @return
     */
    @ApiOperation(tags = {
            "CMC"}, value = "管理员登录", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/managers/login", method = {RequestMethod.POST})
    public BocloudResult<Map<String, Object>> managerLogin(@RequestBody LoginModel loginModel,
                                                           HttpServletRequest request) {
        String account = loginModel.getAccount();
        String password = loginModel.getPassword();
        BocloudResult<Map<String, Object>> loginResult = userService
                .login(account, password, request.getSession().getId(), IpTool.getIP(request), loginModel.getIsManager());
        if (!"admin".equals(account) && loginResult.isFailed()) {
            if (null == loginResult.getData()) {
                String value = redisTemplate.opsForValue().get(RequestContext.Catalog.Manager.name() + account);
                SystemConfig errorNumber = systemConfigRepository.queryByCode("pwdErrorNumber");
                int error = Integer.parseInt(errorNumber.getValue());
                if (!StringUtils.hasText(value)) {
                    //密码操作失败
                    redisTemplate.opsForValue()
                            .set(RequestContext.Catalog.Manager.name() + account, String.valueOf(1), 60, TimeUnit.SECONDS);
                    loginResult = new BocloudResult<>(false, loginResult.getMessage() + ",已操作【1】次");
                } else if (Integer.parseInt(value) >= error) {
                    Result lockResult = managerService.lockByAccount(account);
                    boolean success = lockResult.isSuccess();
                    redisTemplate.delete(RequestContext.Catalog.Manager.name() + account);
                    if (success) {
                        loginResult = new BocloudResult<>(false, "账号或者密码错误已达【" + error + "】次，若管理员账号存在将被冻结，请联系超级管理员解冻");
                    } else {
                        loginResult = new BocloudResult<>(false, "认证服务账号缓存存在问题");
                    }
                } else {
                    redisTemplate.boundValueOps(RequestContext.Catalog.Manager.name() + account).increment(1L);
                    String num = redisTemplate.opsForValue().get(RequestContext.Catalog.Manager.name() + account);
                    loginResult = new BocloudResult<>(false, loginResult.getMessage() + ",已操作【" + num + "】次");
                }
            }
            return loginResult;
        }
        redisTemplate.delete(RequestContext.Catalog.Manager.name() + account);
        return loginResult;
    }

    /**
     * 管理员登出
     *
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {
            "CMC"}, value = "管理员登出", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/managers/logout")
    public Result managerLogout(
            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return managerService.logout(requestContext.getTarget());
    }

    /**
     * 锁定管理员
     *
     * @param account
     * @return
     */
    @ApiOperation(tags = {
            "CMC"}, value = "锁定管理员", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/managers")
    public Result managerLock(@RequestParam(value = Common.ACCOUNT) String account, HttpServletRequest request) {
        return managerService.lockByAccount(account);
    }

    /**
     * 获取key
     *
     * @param apiKey
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取key", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/key")
    public BocloudResult<AccountSecurity> key(@RequestParam(value = "apiKey") String apiKey) {
        return tenantService.apiKey(apiKey);
    }

    /**
     * 租户根据账号获取登录需要信息
     *
     * @param account
     * @return
     */
    @ApiOperation(tags = {
            "CSC"}, value = "获取租户信息", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/tenants/info")
    public BocloudResult<Map<String, Object>> tenantAccountInfo(
            @RequestParam(value = Common.ACCOUNT) String account) {
        return tenantService.tokenLogin(account);
    }

    /**
     * 用户根据根据账号获取登录需要信息
     *
     * @param account
     * @return
     */
    @PostMapping("/users/info")
    @ApiOperation(tags = {
            "CSC"}, value = "获取用户信息", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<Map<String, Object>> userAccountInfo(@RequestParam(value = Common.ACCOUNT) String account) {
        return userService.tokenLogin(account);
    }

    /**
     * 管理员根据根据账号获取登录需要信息
     *
     * @param account
     * @return
     */
    @ApiOperation(tags = {
            "CMC"}, value = "获取管理员信息", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/managers/info")
    public BocloudResult<Map<String, Object>> managerAccountInfo(
            @RequestParam(value = Common.ACCOUNT) String account) {
        return managerService.tokenLogin(account);
    }

    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取登录账号信息详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/login/detail")
    public BocloudResult<Map<String, Object>> loginDetail(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Map<String, Object> resultMap = MapTools.simpleMap("accountCategory", requestContext.getCatalog());
        switch (requestContext.getCatalog()) {
            case Tenant:
                resultMap.put(requestContext.getCatalog().name(),
                        tenantService.detail(requestContext.getTarget()).getData());
                BocloudResult<TenantAccount> accountDetail =
                        cosInternelService.tenantAccountDetail(requestContext.getTarget(), requestContext.getApiKey());
                resultMap.put("tenantAccount", accountDetail.isSuccess() ? accountDetail.getData() : null);
                break;
            default:
                resultMap
                        .put(requestContext.getCatalog().name(), userService.detail(requestContext.getTarget()).getData());
                break;
        }
        Long projectId = Long.parseLong(
                Optional.ofNullable(redisTemplate.opsForValue().get("current_project_" + requestContext.getApiKey()))
                        .orElse("0"));
        resultMap.put("projectId", projectId);
        return new BocloudResult<>(true, resultMap, "获取登录账号信息成功");
    }

    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取登录用户权限", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/users/permissions")
    public BocloudResult loginPermission(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        switch (requestContext.getCatalog()) {
            case Tenant:
                return tenantService.listPermissions(requestContext);
            case Manager:
                return managerService.listPermissions(requestContext);
            case User:
                return userService.listPermissions(requestContext);
            default:
        }
        return new BocloudResult(false, "获取用户信息异常");
    }
}
