package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Role;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.RoleService;
import com.bocloud.sms.model.ApiInfo;
import com.bocloud.sms.model.CloudServiceBean;
import com.bocloud.sms.model.RoleModel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/roles")
@Api(value = "角色管理", tags = "角色管理")
public class RoleController {

    private final RoleService roleService;

    public RoleController(RoleService roleService) {
        this.roleService = roleService;
    }

    /**
     * 查询角色列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取角色列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<Role>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.list(pager, requestContext);
    }

    /**
     * 添加角色
     *
     * @param roleModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建角色", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody RoleModel roleModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.create(roleModel, requestContext);
    }

    /**
     * 修改角色
     *
     * @param roleModel
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "角色修改", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody RoleModel roleModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.modify(id, roleModel, requestContext);
    }

    /**
     * 移除角色
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "角色删除", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.remove(id, requestContext);
    }

    /**
     * 查看角色详细信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "角色详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<Role> detail(@PathVariable(value = Common.ID) Long id) {
        return roleService.detail(id);
    }

    /**
     * 角色授权
     *
     * @param id
     * @param ids
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "角色授权", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}")
    public Result accredit(@PathVariable(value = Common.ID) Long id, @RequestBody List<Long> ids,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.accredit(id, ids, requestContext);
    }

    /**
     * 查询角色的权限列表
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取权限列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/permissions")
    public BocloudResult<List<CloudServiceBean>> permissions(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "menu", required = false) boolean menu,
                                                             @RequestParam(value = "tenant", required = false) boolean tenant, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.listPermissions(id, menu, tenant, requestContext);
    }

    /**
     * 获取请求用户角色信息（用于boot comment 获取角色Api权限）
     *
     * @param requestContext
     * @return
     */
    @GetMapping("/ids")
    public BocloudResult<List<Long>> roleIds(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.roleIds(requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "API授权", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/apis")
    public Result accreditRole(@PathVariable(value = Common.ID) Long id, @RequestBody List<ApiInfo> apis) {
        return roleService.accreditRole(id, apis);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询API权限列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/apis")
    public BocloudResult<JSONArray> getModuleList(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "tags", required = false) String tags) {
        return roleService.getApiPermission(id, tags);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询指定角色下的用户", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/users")
    public BocloudResult<GridBean<User>> listUsersByRoleId(Pager pager,@PathVariable (value = Common.ID)Long id){
        return roleService.listUsersByRoleId(pager,id);
    }
}
