package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.UserService;
import com.bocloud.sms.model.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/users")
@Api(value = "用户管理", tags = "用户管理")
public class UserController {

    private final Logger logger = LoggerFactory.getLogger(UserController.class);
    private final UserService userService;

    public UserController(UserService userService) {
        this.userService = userService;
    }

    /**
     * 获取用户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询用户列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<User>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.list(pager, requestContext);
    }

    /**
     * 创建用户
     *
     * @param userBean
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建用户", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody UserBean userBean,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.create(userBean, requestContext);
    }

    /**
     * 修改用户
     *
     * @param userBean
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "用户修改", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@RequestBody UserBean userBean,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext, @PathVariable(value = Common.ID) Long id) {
        return userService.modify(id, userBean, requestContext);
    }

    /**
     * 移除用户
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除用户", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.remove(id, requestContext);
    }

    /**
     * 查看用户详细信息
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "用户详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<User> detail(@PathVariable(value = Common.ID) Long id) {
        return userService.detail(id);
    }


    /**
     * 查询用户账号是否存在
     *
     * @param account
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询用户账号是否存在", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{account}/exist")
    public Result exist(@PathVariable(value = Common.ACCOUNT) String account) {
        return userService.exist(account);
    }

    /**
     * 查看租户详情
     *
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "用户详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/detail")
    public BocloudResult<User> detail(@Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.detail(requestContext.getTarget());
    }


    /**
     * 修改用户状态
     *
     * @param id
     * @param action
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "修改用户状态", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}")
    public Result updateStatus(@PathVariable(value = Common.ID) Long id,
                               @RequestBody ActionModel action,
                               @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.updateStatus(id, action.getAction(), requestContext);
    }

    /**
     * 修改用户密码
     *
     * @param id             用户id
     * @param passwordModel  旧密码
     * @param requestContext 请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "修改用户密码", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/pwd")
    public Result changePwd(@PathVariable(value = Common.ID) Long id,
                            @RequestBody PasswordModel passwordModel,
                            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.changePwd(id, passwordModel.getOldPassword(), passwordModel.getPassword(), requestContext);
    }

    /**
     * 重置用户密码
     *
     * @param id             用户id
     * @param passwordModel  密码
     * @param requestContext 请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "重置用户密码", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/reset")
    public Result resetPwd(@PathVariable(value = Common.ID) Long id,
                           @RequestBody PasswordModel passwordModel,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.resetPwd(id, passwordModel.getPassword(), requestContext);
    }

    /**
     * 用户授权
     *
     * @param id             用户id
     * @param roleIds        角色id
     * @param requestContext 请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "用户授权角色", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/accredit")
    public Result accredit(@PathVariable(value = Common.ID) Long id,
                           @RequestBody List<Long> roleIds,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.accredit(id, roleIds, requestContext);
    }

    /**
     * 检查用户是否异地登录
     *
     * @param id
     * @param request
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "检查用户是否登录", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/status")
    public Result checkUser(@PathVariable(value = Common.ID) Long id, HttpServletRequest request) {
        return userService.checkUser(id, request.getHeader(Common.TOKEN));
    }

    /**
     * 用户的角色列表
     *
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取角色列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/roles")
    public BocloudResult<List<Role>> roles(@PathVariable(value = Common.ID) Long id,
                                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listRoles(id, requestContext);
    }

    /**
     * 获取key
     *
     * @param apiKey
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取key", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/security")
    public BocloudResult<AccountSecurity> secKey(@RequestParam(value = "apiKey") String apiKey) {
        return userService.getAccountSecurity(apiKey);
    }

    /**
     * 批量操作用户
     *
     * @param batchAction
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "批量操作用户", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping
    public Result batchLock(@RequestBody BatchActionModel batchAction, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        switch (batchAction.getAction()) {
            case lock:
                return userService.batchLock(batchAction.getIds(), requestContext);
            case active:
                return userService.batchActive(batchAction.getIds(), requestContext);
            case remove:
                return userService.batchRemove(batchAction.getIds(), requestContext);
            default:
                return new Result(false, "不支持这种操作");
        }
    }

    /**
     * 用户移除所属租户
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "用户移除所属租户", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}/tenants")
    public Result unBindTenants(@PathVariable Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.unBindTenants(id, requestContext);
    }

    /**
     * 登录记录查询
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "登录记录查询", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/track")
    public BocloudResult<GridBean<LoginRecord>> records(Pager pager) {
        return this.userService.list(pager);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加收藏", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/favorites/{id}")
    public Result addFavorite(@PathVariable("id") Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.addFavorite(id, requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除收藏", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/favorites/{id}")
    public Result cancelFavorite(@PathVariable("id") Long serviceId, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.cancelFavorite(serviceId, requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取收藏夹", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/favorites")
    public BocloudResult<List<CloudServiceBean>> listFavorite(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listFavorite(requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务目录", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/services")
    public BocloudResult<List<ServiceMenuBean>> listServiceMenu(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listServiceMenu(requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改收藏夹排序", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/favorites/order")
    public Result changePriority(@RequestBody List<PriorityModel> priorityModels, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.changePriority(priorityModels, requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取用户角色授权服务", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/roles/services")
    public BocloudResult<List<CloudService>> getUserService(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.getUserServices(requestContext);
    }


    /**
     * 获取某个用户下的所有项目
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取项目列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}/projects")
    public BocloudResult<List<Project>> projects(@PathVariable(value = Common.ID) Long id) {
        return userService.projects(id);
    }

    /**
     * 导出用户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated
    @ApiOperation(tags = {"CMC", "CSC"}, value = "导出项目列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                       HttpServletRequest request, HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        Result export = userService.export(paramList, requestContext, response);
        Assert.isTrue(export.isSuccess(),"用户列表导出失败");
    }

    @ApiOperation(value = "下载用户列表模板", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletRequest request, HttpServletResponse response,
                           @RequestParam(value = "tenantId", required = false) Long tenantId,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Result result = userService.importTemp("用户列表", response);
        Assert.isTrue(result.isSuccess(), "用户模板下载失败");
    }

    @ApiOperation(value = "导入用户列表", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext,
                             @RequestParam(value = "tenantId", required = false) Long tenantId, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return userService.importData(multipartFile, tenantId, requestContext);
    }
}
