package com.bocloud.sms.service;

import com.bocloud.common.enums.BaseStatus;
import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.Manager;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.DepartmentService;
import com.bocloud.sms.model.DepartmentModel;
import com.bocloud.sms.repository.DepartmentRepository;
import com.bocloud.sms.repository.UserRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 组织机构Service接口实现
 *
 * @author tyl
 */
@Service("departmentService")
public class DepartmentServiceImpl implements DepartmentService {

    private final Logger logger = LoggerFactory.getLogger(DepartmentServiceImpl.class);
    private final DepartmentRepository departmentRepository;
    private final UserRepository userRepository;
    private final LockFactory lockFactory;

    public DepartmentServiceImpl(DepartmentRepository departmentRepository, UserRepository userRepository, LockFactory lockFactory) {
        this.departmentRepository = departmentRepository;
        this.userRepository = userRepository;
        this.lockFactory = lockFactory;
    }

    /**
     * 新建组织机构
     *
     * @param model          组织机构信息
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result create(DepartmentModel model, RequestContext requestContext) throws InternalServerException {
        try {
            Department department = new Department();
            BeanUtils.copyProperties(model, department);
            department.setCreatorId(requestContext.getTarget());
            if (null == requestContext.getTenant()) {
                department.setTenantId(0L);
            } else {
                department.setTenantId(requestContext.getTenant());
            }
            department.setStatus(BaseStatus.NORMAL.name());
            //判断组织机构信息是否存在
            BocloudResult<Department> result = exists(department.getName(), department.getTenantId());
            Assert.isTrue(result.isFailed(), "此名称已存在!");
            departmentRepository.save(department);
            return new Result(true, "添加组织机构成功!");
        } catch (Exception e) {
            logger.error("create department fail:", e);
            throw new InternalServerException("创建组织机构失败！" + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 修改组织机构信息
     *
     * @param id
     * @param departmentModel 组织机构信息json串
     * @param requestContext  请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result modify(Long id, DepartmentModel departmentModel, RequestContext requestContext) throws InternalServerException {
        String path = Department.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Department depart = departmentRepository.query(id);
            Assert.notNull(depart, "组织机构信息不存在！");
            //判断组织机构信息是否存在
            BocloudResult<Department> result = exists(depart.getName(), depart.getTenantId());
            if (result.isSuccess()) {
                Department dept = result.getData();
                Assert.isTrue(dept.getId().equals(depart.getId()), "此名称已存在!");
            }
            depart.setMenderId(requestContext.getTarget());
            BeanUtils.copyProperties(departmentModel, depart);
            boolean flag = departmentRepository.update(depart);
            return flag ? new Result(true, "修改组织机构成功") : new Result(false, "修改组织机构失败");
        } catch (Exception e) {
            logger.error("modify department fail:", e);
            throw new InternalServerException("修改组织机构信息失败！" + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 删除组织机构信息
     *
     * @param id             组织机构id
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result remove(Long id, RequestContext requestContext) throws InternalServerException {
        String path = Department.class.getSimpleName() + "_" + id;
        int result;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时！");
            Department depart = departmentRepository.query(id);
            Assert.notNull(depart, "数据不存在。");
            List<Department> children = departmentRepository.list(id);
            Assert.isTrue(children.isEmpty(), "此组织机构存在子机构。");
            result = departmentRepository.delete(id, requestContext.getTarget());
            return result > 0 ? new Result(true, "删除组织机构成功。") : new Result(false, "删除组织机构失败。");
        } catch (Exception e) {
            logger.error("remove department fail:", e);
            throw new InternalServerException("删除组织机构信息失败！" + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 查看组织机构详情
     *
     * @param id 组织机构id
     * @return
     */
    @Override
    public BocloudResult<Department> detail(Long id) {
        try {
            Department department = departmentRepository.query(id);
            Assert.notNull(department, "组织机构数据不存在");
            return new BocloudResult<>(true, department, "查询组织机构详情成功");
        } catch (Exception e) {
            logger.error("get department detail fail:", e);
            return new BocloudResult<>(false, "查询组织机构详情失败！" + e.getMessage());
        }
    }

    /**
     * 获取组织机构列表
     *
     * @param parentId       父节点id
     * @param requestContext 请求对象
     * @return
     */
    @Override
    public BocloudResult<List<Department>> list(Long parentId, RequestContext requestContext) {
        try {
            List<Department> departments = departmentRepository.list(parentId);
            Map<Long, List<Department>> departmentMap = departmentRepository.listAll().stream().peek(dept -> {
                if (dept.getParentId() == null) {
                    dept.setParentId(0L);
                }
            }).collect(Collectors.groupingBy(Department::getParentId));
            for (Department department : departments) {
                getChildren(department, departmentMap);
            }
            return new BocloudResult<>(true, departments, "查询组织机构成功");
        } catch (Exception e) {
            logger.error("list department fail:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    /**
     * 根据层级获取组织机构列表
     *
     * @param parentId       父节点id
     * @param requestContext 请求对象
     * @return
     */
    @Override
    public BocloudResult<List<Department>> listByParentId(Long parentId, RequestContext requestContext) {
        try {
            List<Department> departments = departmentRepository.list(parentId);
            Map<Long, List<Department>> departmentMap = departmentRepository.listAll().stream().peek(dept -> {
                if (dept.getParentId() == null) {
                    dept.setParentId(0L);
                }
            }).collect(Collectors.groupingBy(Department::getParentId));
            for (Department department : departments) {
                if (departmentMap.containsKey(department.getId())){
                    department.setIsParent(true);
                } else {
                    department.setIsParent(false);
                }
            }
            return new BocloudResult<>(true, departments, "查询组织机构成功");
        } catch (Exception e) {
            logger.error("list department fail:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    private void getChildren(Department department, Map<Long, List<Department>> deptMap) {
        List<Department> children = deptMap.get(department.getId());
        if (null != children) {
            for (Department child : children) {
                getChildren(child, deptMap);
            }
            department.setChildren(children);
        }
    }

    /**
     * 判断组织机构是否存在
     *
     * @param name     组织机构名称
     * @param tenantId 租户id
     * @return
     */
    @Override
    public BocloudResult<Department> exists(String name, Long tenantId) {
        try {
            Department department = departmentRepository.exists(name, tenantId);
            if (null == department) {
                return new BocloudResult<>(false, "不存在!");
            } else {
                return new BocloudResult<>(true, department, "存在!");
            }
        } catch (Exception e) {
            logger.error("get department failure:", e);
            return new BocloudResult<>(false, "不存在!");
        }
    }

    @Override
    public BocloudResult<List<User>> listDeptManager(Long id, RequestContext requestContext) {
        try {
            List<User> users = userRepository.listByDepartment(id);
            return new BocloudResult<>(true, users, "查询成功");
        } catch (Exception e) {
            logger.error("list department users error,", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }
}
