package com.bocloud.sms.service;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.interfaces.DictService;
import com.bocloud.sms.model.DictionaryBean;
import com.bocloud.sms.repository.DictionaryRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 字典服务实现类
 *
 * @author dmw
 */
@Service("dictService")
public class DictServiceImpl implements DictService {

    private static final Logger logger = LoggerFactory.getLogger(DictServiceImpl.class);
    private final DictionaryRepository dictionaryRepository;
    private final LockFactory lockFactory;

    public DictServiceImpl(DictionaryRepository dictionaryRepository, LockFactory lockFactory) {
        this.dictionaryRepository = dictionaryRepository;
        this.lockFactory = lockFactory;
    }

    @Override
    public BocloudResult<List<Dictionary>> list(Long pid) {
        try {
            Map<Long, List<Dictionary>> dictionaryMap = dictionaryRepository.list().stream().collect(Collectors.groupingBy(Dictionary::getPid));
            List<Dictionary> dictionaries = dictionaryRepository.dictionaries(pid);
            for (Dictionary dictionary : dictionaries) {
                getChildren(dictionary, dictionaryMap);
            }
            return new BocloudResult<>(true, dictionaries, "查询成功");
        } catch (Exception e) {
            logger.error("list dictionary fail:", e);
            return new BocloudResult<>(false, "查询失败", "", "");
        }
    }


    @Override
    public BocloudResult<List<Dictionary>> listChild(String value) {
        try {
            Dictionary dictionary = dictionaryRepository.queryFistDic(value);
            if (dictionary == null) {
                dictionary = dictionaryRepository.queryDic(value);
            }
            if (null == dictionary) {
                return new BocloudResult<>(true, "未找到【" + value + "】字典");
            }

            List<Dictionary> dicts = this.dictionaryRepository.dictionaries(dictionary.getId());
            return new BocloudResult<>(true, dicts, "success");
        } catch (Exception e) {
            logger.error("List Children Dictionary exception:{}", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<Dictionary> create(Dictionary dictionary) {
        try {
            boolean existValue = dictionaryRepository.existValue(dictionary.getId(), dictionary.getValue());
            if (existValue) {
                return new BocloudResult<>(false, "内容已存在，添加失败");
            }
            dictionaryRepository.save(dictionary);
            return new BocloudResult<>(true, dictionary, "保存成功!");
        } catch (Exception e) {
            logger.error("create dictionary failure:", e);
            return new BocloudResult<>(false, "保存失败!");
        }
    }

    @Override
    public Result remove(Long id) {
        String path = Dictionary.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Dictionary dictionary = dictionaryRepository.query(id);
            if (null == dictionary) {
                return new BocloudResult<>(false, "数据不存在!");
            }
            Map<Long, List<Dictionary>> dictionaryMap = dictionaryRepository.dictionaries(null).stream().collect(Collectors.groupingBy(Dictionary::getPid));
            List<Long> childrenIds = getChildrenIds(dictionary, dictionaryMap);
            childrenIds.add(dictionary.getId());
            boolean result = dictionaryRepository.batchRemove(childrenIds);
            return result ? new BocloudResult<>(true, "删除成功!") : new BocloudResult<>(false, "删除失败!");
        } catch (Exception e) {
            logger.error("Remove dictionary error:", e);
            return new BocloudResult<>(false, "删除失败");
        }
    }

    private List<Long> getChildrenIds(Dictionary dictionary, Map<Long, List<Dictionary>> dictionaryMap) {
        List<Long> childrenIds = new ArrayList<>();
        List<Dictionary> children = dictionaryMap.get(dictionary.getId());
        if (children != null) {
            for (Dictionary child : children) {
                childrenIds.add(child.getId());
                List<Long> childIds = getChildrenIds(child, dictionaryMap);
                if (!childIds.isEmpty()) {
                    childrenIds.addAll(childIds);
                }
            }
        }
        return childrenIds;
    }

    @Override
    public BocloudResult<Dictionary> modify(DictionaryBean bean) {
        String path = Dictionary.class.getSimpleName() + "_" + bean.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Dictionary dictionary = dictionaryRepository.query(bean.getId());
            if (null == dictionary) {
                return new BocloudResult<>(false, "数据不存在!");
            }
            BeanUtils.copyProperties(bean, dictionary);
            dictionaryRepository.update(dictionary);
            return new BocloudResult<>(true, dictionary, "修改成功!");
        } catch (Exception e) {
            logger.error("Modify server fail:", e);
            return new BocloudResult<>(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<List<Dictionary>> childTree(String value) {
        try {
            Dictionary dictionary = dictionaryRepository.queryFistDic(value);
            if (dictionary == null) {
                dictionary = dictionaryRepository.queryDic(value);
            }
            Assert.notNull(dictionary, "未找到value为[" + value + "]字典");
            List<Dictionary> result = this.dictionaryRepository.dictionaries(dictionary.getId());
            Map<Long, List<Dictionary>> dictionaryMap = dictionaryRepository.list().stream().collect(Collectors.groupingBy(Dictionary::getPid));
            for (Dictionary dic : result) {
                getChildren(dic, dictionaryMap);
            }
            return new BocloudResult<>(true, result, "success");
        } catch (Exception e) {
            logger.error("List Children Dictionary exception:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private void getChildren(Dictionary dictionary, Map<Long, List<Dictionary>> dicMap) {
        List<Dictionary> children = dicMap.get(dictionary.getId());
        if (null != children) {
            children.sort((x, y) -> Long.compare(x.getWeight(), y.getWeight()));
            for (Dictionary child : children) {
                getChildren(child, dicMap);
            }
            dictionary.setChildren(children);
        }
    }
}