package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.encrypt.Encryptor;
import com.bocloud.common.encrypt.SHAEncryptor;
import com.bocloud.common.enums.BaseStatus;
import com.bocloud.common.enums.BocloudStatus;
import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.*;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.license.LicenseDaemon;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.ManagerService;
import com.bocloud.sms.model.ActionModel;
import com.bocloud.sms.model.ManagerModel;
import com.bocloud.sms.model.PermissionBean;
import com.bocloud.sms.repository.*;
import com.bocloud.sms.utils.PermitComparator;
import com.google.common.collect.Lists;
import io.jsonwebtoken.SignatureAlgorithm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.bocloud.sms.service.utils.LocalDateTimeUtil.localDateTimeFormat;

/**
 * @author tyl
 * @time 2020/2/4 周二 12:50
 */
@Service("managerService")
public class ManagerServiceImpl implements ManagerService {

    private final Logger logger = LoggerFactory.getLogger(ManagerService.class);

    private final StringRedisTemplate redisTemplate;
    private final ManagerRepository managerRepository;
    private final PermissionRepository permissionRepository;
    private final RoleRepository roleRepository;
    private final AccountSecurityRepository securityRepository;
    private final DepartmentRepository departmentRepository;
    private final LockFactory lockFactory;
    private final LicenseDaemon licenseDaemon;
    private final ApiPermissionRepository apiPermissionRepository;
    private final LoginRecordRepository loginRecordRepository;
    private final ManagerRoleRepository managerRoleRepository;

    public ManagerServiceImpl(StringRedisTemplate redisTemplate, ManagerRepository managerRepository,
                              PermissionRepository permissionRepository, TenantRepository tenantRepository,
                              RoleRepository roleRepository, AccountSecurityRepository securityRepository,
                              DepartmentRepository departmentRepository, LockFactory lockFactory, CurrentService currentService, ApiPermissionRepository apiPermissionRepository, LoginRecordRepository loginRecordRepository, ManagerRoleRepository managerRoleRepository) {
        this.redisTemplate = redisTemplate;
        this.managerRepository = managerRepository;
        this.permissionRepository = permissionRepository;
        this.roleRepository = roleRepository;
        this.securityRepository = securityRepository;
        this.departmentRepository = departmentRepository;
        this.lockFactory = lockFactory;
        this.licenseDaemon = LicenseDaemon.instance(currentService);
        this.apiPermissionRepository = apiPermissionRepository;
        this.loginRecordRepository = loginRecordRepository;
        this.managerRoleRepository = managerRoleRepository;
    }

    /**
     * 管理员登录
     *
     * @param account
     * @param password
     * @param sessionId
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public BocloudResult<Map<String, Object>> login(String account, String password, String sessionId, String requestIp) throws InternalServerException {
        LoginRecord log = new LoginRecord();
        log.setRequestIp(requestIp);
        log.setResponseIp(NicTools.getIp());
        log.setAccount(account);
        log.setCategory(LoginRecord.Category.Manager);
        log.setGmtCreate(new Date());
        // 判断管理员是否存在
        Manager manager;
        try {
            manager = managerRepository.getByAccount(account);
        } catch (Exception e) {
            handleLoginLog(false, "账号或密码错误", log);
            logger.error("Get Manager fail by account", e);
            return new BocloudResult<>(false, "账号或密码错误");
        }
        if (null == manager) {
            handleLoginLog(false, "账号或密码错误", log);
            return new BocloudResult<>(false, "账号或密码错误");
        }
        if (manager.getStatus().equals(BaseStatus.ABNORMAL.name())) {
            handleLoginLog(false, "账号已冻结", log);
            return new BocloudResult<>(false, "冻结管理员无法登录!");
        }
        if (manager.getStatus().equals(BocloudStatus.Basic.EXPIRED.name())) {
            handleLoginLog(false, "账号已过期", log);
            return new BocloudResult<>(false, "账号已过期，请联系超级管理员!");
        }
        AccountSecurity security;
        try {
            security = securityRepository.getByTarget(manager.getId(), Catalog.Manager);
        } catch (Exception e) {
            handleLoginLog(false, "账号或密码错误", log);
            logger.error("Query Random fail:", e);
            return new BocloudResult<>(false, "账号或密码错误");
        }
        if (null == security) {
            handleLoginLog(false, "账号或密码错误", log);
            logger.warn("Security is null");
            return new BocloudResult<>(false, "账号或密码错误");
        }
        Encryptor encryptor = new AESEncrptor();
        String result;
        try {
            // 对前端传过来的密码进行解密
            password = encryptor.decrypt(password.trim(), null);
            if (!StringUtils.hasText(password)) {
                handleLoginLog(false, "账号或密码错误", log);
                return new BocloudResult<>(false, "账号或密码错误");
            }
            // 密码加密
            encryptor = new SHAEncryptor();
            result = encryptor.encrypt(password, security.getSalt());
        } catch (Exception e) {
            logger.error("Get error message:", e);
            handleLoginLog(false, "账号或密码错误", log);
            return new BocloudResult<>(false, "账号或密码错误");
        }
        // 判断是否密码正确
        if (null == result) {
            logger.warn("Wrong manager account or password");
            handleLoginLog(false, "账号或密码错误", log);
            return new BocloudResult<>(false, "账号或密码错误");
        }
        if (!result.equals(manager.getPassword())) {
            logger.warn("Wrong manager account or password");
            handleLoginLog(false, "账号或密码错误", log);
            return new BocloudResult<>(false, "账号或密码错误");
        }
        //判断管理员账号是否在有效期内
        Date expireDate = manager.getExpiredDate();
        if (expireDate != null) {
            if (!expireDate.after(new Date())) {
                logger.warn("the manager account is expired");
                handleLoginLog(false, "该管理员账号已过期", log);
                return new BocloudResult<>(false, "该管理员账号已过期，请联系超级管理进行修改");
            }
        }
        try {
            // 更新管理员登录状态
            manager.setLoginStatus(true);
            manager.setSessionId(sessionId);
            //获取上一次登录时间用于显示 并更新数据库上一次登录时间为现在
            Date lastLoginDate = manager.getLastLoginDate();
            manager.setLastLoginDate(new Date());
            managerRepository.update(manager);
            manager.setLastLoginDate(lastLoginDate);
        } catch (Exception e) {
            handleLoginLog(false, "更新用户登录状态失败", log);
            logger.error("Update manager login status fail：", e);
            throw new InternalServerException("更新管理员登录状态失败", e, e.getMessage());
        }
        //刷新权限缓存
        List<Long> roleIds = roleRepository.listManagerApiRole(manager.getId()).stream().map(Role::getId).collect(Collectors.toList());
        List<ApiPermission> apiPermissions = apiPermissionRepository.listRoleApiPermission(roleIds);
        Map<Long, List<ApiPermission>> rolePermissionMap = apiPermissions.stream().collect(Collectors.groupingBy(ApiPermission::getRoleId));
        for (Map.Entry<Long, List<ApiPermission>> rolePermissionEntry : rolePermissionMap.entrySet()) {
            Long roleId = rolePermissionEntry.getKey();
            Map<String, List<ApiPermission>> permissionMap = rolePermissionEntry.getValue().stream().collect(Collectors.groupingBy(ApiPermission::getModule));
            for (Map.Entry<String, List<ApiPermission>> permission : permissionMap.entrySet()) {
                String module = permission.getKey();
                List<String> patterns = permission.getValue().stream().map(s -> new String(Base64.getDecoder().decode(s.getPattern()))).collect(Collectors.toList());
                if (!patterns.isEmpty()) {
                    redisTemplate.opsForHash().put("api_permission_" + roleId, module, JSONObject.toJSONString(patterns));
                }
            }
        }
        // 构建返回数据
        Map<String, Object> map = MapTools.simpleMap("manager", manager);
        map.put(Common.API_KEY, security.getApiKey());
        map.put("accountCategory", Catalog.Manager.name());
        //构建token
        Map<String, Object> header = MapTools.simpleMap("alg", SignatureAlgorithm.HS256.getValue());
        header.put("typ", "JWT");
        Map<String, Object> payload = MapTools.simpleMap(Common.ACCOUNT, manager.getAccount());
        payload.put("catalog", Catalog.Manager.name());
        payload.put(Common.API_KEY, security.getApiKey());
        payload.put(Common.UUID, manager.getId());
        payload.put(Common.NAME, manager.getName());
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MONTH, 3);
        // 加密jwt生成token并存入session和cookie
        String token = Tokens.generate(header, payload, calendar.getTime());
        map.put(Common.TOKEN, token);
        String tokenKey = "token_" + Catalog.Manager.name() + "_" + manager.getId();
        this.redisTemplate.delete(tokenKey + "_o");
        HashMap<String, Object> tokenJson = new HashMap<>();
        tokenJson.put("token", token);
        tokenJson.put("refreshTime", localDateTimeFormat(LocalDateTime.now().plusMinutes(10)));
        redisTemplate.opsForHash().putAll(tokenKey + "_n", tokenJson);
        this.redisTemplate.expire(tokenKey + "_n", 30L, TimeUnit.MINUTES);
        handleLoginLog(true, "管理员登录成功", log);
        return new BocloudResult<>(true, map, "管理员登录成功");
    }

    /**
     * 保存登录日志
     *
     * @param status 登录状态
     * @param detail 登录详情
     * @param record 登录日志对象
     */
    private void handleLoginLog(boolean status, String detail, LoginRecord record) {
        record.setStatus(status);
        record.setDetail(detail);
        try {
            this.save(record);
        } catch (Exception e) {
            logger.error("save login record error");
        }
    }

    /**
     * 递归查询父菜单
     *
     * @param allPermissionMap 所有权限集合
     * @param parents          返回目标父权限集合
     * @param permission       需查询的权限
     * @return
     */
    private List<Permission> getParents(Map<Long, Permission> allPermissionMap, List<Permission> parents,
                                        Permission permission) {
        Long parentId = permission.getParentId();
        //当前权限没有父权限id(递归到顶层) ，则添加到父权限集合，到达递归出口，返回父权限集合
        if (null == parentId || 0L == parentId) {
            parents.add(permission);
            return parents;
        } else {
            //当前权限有父权限id，则获取当前权限的父权限，添加到返回集合，并再次递归
            try {
                Permission parent = allPermissionMap.get(parentId);
                if (null!=parent){
                    parents.add(parent);
                }
                return this.getParents(allPermissionMap, parents, parent);
            } catch (Exception e) {
                logger.error("get Permission failure:", e);
                return null;
            }
        }
    }


    /**
     * 对管理员权限进行递归排序
     *
     * @param pid             父权限id
     * @param permissionBeans 权限集合
     * @return
     */
    private List<PermissionBean> getPermissions(Long pid, List<PermissionBean> permissionBeans) {
        List<PermissionBean> newPermissions = Lists.newArrayList();
        for (PermissionBean permissionBean : permissionBeans) {
            //目标权限id等于父权限id，则进入递归获取其子权限集合
            if (null != permissionBean.getParentId() && permissionBean.getParentId().equals(pid)) {
                List<PermissionBean> children = getPermissions(permissionBean.getId(), permissionBeans);
                //子权限不为空时则对权限进行排序后set到父权限下
                if (!ListTool.isEmpty(children)) {
                    children.sort(new PermitComparator());
                    permissionBean.setChildren(children);
                } else {
                    //子权限为空
                    permissionBean.setChildren(null);
                }
                //添加到返回目标集合
                newPermissions.add(permissionBean);
            }
        }
        newPermissions.sort(new PermitComparator());
        return newPermissions;
    }

    /**
     * 管理员退出登录
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result logout(Long id) throws InternalServerException {
        try {
            Manager manager = managerRepository.query(id);
            if (manager != null) {
                manager.setLoginStatus(false);
                manager.setSessionId("");
                //修改管理员登录状态
                managerRepository.update(manager);
                String tokenKey = Common.TOKEN + "_" + Catalog.Manager.name() + "_" + id;
                redisTemplate.delete(tokenKey + "_n");
                redisTemplate.delete(tokenKey + "_o");
            }
            return new Result(true, "管理员退出成功");
        } catch (Exception e) {
            logger.error("get manager fail:", e);
            throw new InternalServerException("获取管理员信息失败!", e, e.getMessage());
        }
    }

    /**
     * 创建管理员
     *
     * @param managerModel
     * @param requestContext
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result create(ManagerModel managerModel, RequestContext requestContext) throws InternalServerException {
        Manager manager = new Manager();
        BeanUtils.copyProperties(managerModel, manager);
        manager.setCreatorId(requestContext.getTarget());
        // 初始化密码
        String salt = UUID.randomUUID().toString();
        Encryptor encryptor = new SHAEncryptor();
        try {
            Manager existManager = managerRepository.getByAccount(manager.getAccount());
            Assert.isNull(existManager, "账号已存在！");
            String password = encryptor.encrypt(new AESEncrptor().decrypt(manager.getPassword().trim(), null), salt);
            manager.setPassword(password);
            //关键信息去空
            manager.setAccount(manager.getAccount().trim());
            manager.setName(manager.getName().trim());
            manager.setStatus(BaseStatus.NORMAL.name());
            manager.setLoginStatus(false);
            manager.setLastPwdModifyDate(new Date());
            manager.setOrigin("SYSTEM");
            managerRepository.save(manager);
            String account = manager.getAccount();
            Manager cacheManager = managerRepository.getByAccount(account);
            redisTemplate.opsForValue().set(Catalog.Manager.name() + "_" + cacheManager.getId(), account);
            redisTemplate.opsForValue().set(Catalog.Manager.name() + "_Name_" + cacheManager.getId(), cacheManager.getName());
        } catch (Exception e) {
            logger.error("create manager fail:", e);
            throw new InternalServerException("添加管理员失败！" + e.getMessage(), e, e.getMessage());
        }
        // 构建管理员随机数对象
        AccountSecurity security = new AccountSecurity();
        String data = Long.toString(System.currentTimeMillis());
        String apiKey;
        String secKey;
        try {
            apiKey = encryptor.encrypt(data, Common.API_KEY);
            secKey = encryptor.encrypt(data, Common.SEC_KEY);
        } catch (Exception e) {
            logger.error("Get error message:", e);
            throw new InternalServerException("添加管理员失败", e, e.getMessage());
        }
        security.setTargetId(manager.getId());
        security.setCategory(AccountSecurity.Category.Manager);
        security.setSalt(salt);
        security.setApiKey(apiKey);
        security.setSecKey(secKey);
        security.setTenantId(0L);
        boolean success;
        try {
            success = securityRepository.baseSave(security);
        } catch (Exception e) {
            logger.error("Manager random add fail:", e);
            throw new InternalServerException("添加管理员失败", e, e.getMessage());
        }
        return success ? new Result(true, "添加管理员成功") : new Result(false, "添加管理员失败");
    }

    /**
     * 更新管理员
     *
     * @param managerModel
     * @param requestContext
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result modify(Long id, ManagerModel managerModel, RequestContext requestContext) throws InternalServerException {
        managerModel.setName(managerModel.getName().trim());
        Assert.notNull(managerModel, "管理员对象不能为空！");
        String path = Manager.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时！");
            Manager manager = managerRepository.query(id);
            Assert.notNull(manager, "管理员数据不存在！");
            BeanUtils.copyProperties(managerModel, manager);
            manager.setMenderId(requestContext.getTarget());
            if (manager.getExpiredDate() != null && manager.getExpiredDate().before(new Date())) {
                manager.setStatus(BocloudStatus.Basic.EXPIRED.name());
            } else {
                if (manager.getStatus().equals(BocloudStatus.Basic.EXPIRED.name())) {
                    manager.setStatus(BocloudStatus.Basic.NORMAL.name());
                }
            }
            boolean result = managerRepository.update(manager);
            //缓存更新管理员姓名
            redisTemplate.opsForValue().set(Catalog.Manager.name() + "_Name_" + manager.getId(), manager.getName());
            return result ? new Result(true, "修改成功") : new Result(false, "修改失败");
        } catch (Exception e) {
            logger.error("modify manager fail:", e);
            throw new InternalServerException("修改管理员失败！" + e.getMessage(), e);
        }
    }

    /**
     * 删除管理员
     *
     * @param id
     * @param requestContext
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result remove(Long id, RequestContext requestContext) throws InternalServerException {
        String path = Manager.class.getSimpleName() + "_" + id;
        Long userId = requestContext.getTarget();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时！");
            Manager manager = managerRepository.query(id);
            Assert.notNull(manager, "管理员数据不存在！");
            Assert.isTrue(!id.equals(userId), "不能删除登录的管理员！");
            // 删除管理员和角色的对应关系
            managerRepository.unBindRole(id);
            managerRepository.delete(id, userId);
            securityRepository.delete(id, Catalog.Manager);
            //删除缓存中对应的管理员账号信息
            redisTemplate.delete(Catalog.Manager + "_" + manager.getId());
            redisTemplate.delete(Catalog.Manager + "_Name_" + manager.getId());
            return new Result(true, "删除管理员成功");
        } catch (Exception e) {
            logger.error("remove manager fail:", e);
            throw new InternalServerException("删除管理员失败," + e.getMessage(), e, e.getMessage());
        }
    }

    @Override
    @Transactional
    public Result updateStatus(Long id, ActionModel.Action action, RequestContext requestContext) throws InternalServerException {
        String path = Manager.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Manager manager = managerRepository.query(id);
            Assert.notNull(manager, "管理员信息不存在！");
            Long managerId = requestContext.getTarget();
            switch (action) {
                case lock:
                    Assert.isTrue(!id.equals(managerId), "用户不能冻结当前登录的管理员。");
                    boolean result = managerRepository.lock(id, managerId);
                    this.logout(id);
                    return result ? new Result(true, "冻结用户成功") : new Result(false, "冻结用户失败");
                case active:
                    return managerRepository.active(id, managerId) ? new Result(true, "解冻管理员成功") : new Result(false, "解冻用户失败");
                case logout:
                default:
                    return new Result(false, "不支持的操作");
            }
        } catch (Exception e) {
            logger.error("update manager status fail:", e);
            return BocloudResult.FAILED("修改管理员状态失败," + e.getMessage());
        }
    }

    @Override
    public Result changePwd(Long id, String oldPassword, String password, RequestContext requestContext) {
        String path = Manager.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Manager manager = managerRepository.query(id);
            Assert.notNull(manager, "管理员信息不存在！");
            String salt = UUID.randomUUID().toString();
            // 加密
            SHAEncryptor sha = new SHAEncryptor();
            Encryptor encryptor = new AESEncrptor();
            //对原密码解密，并校验。
            oldPassword = encryptor.decrypt(oldPassword.trim(), null);
            Assert.isTrue(StringUtils.hasText(oldPassword), "旧密码不存在");
            Result checkResult = this.check(id, oldPassword);
            if (checkResult.isFailed()) {
                return checkResult;
            }
            // 对前端传过来的密码进行解密
            password = encryptor.decrypt(password.trim(), null);
            Assert.isTrue(StringUtils.hasText(password), "新密码不存在。");
            String encrypt = sha.encrypt(password, salt);
            manager.setPassword(encrypt);
            manager.setLastPwdModifyDate(new Date());
            managerRepository.update(manager);
            // 准备随机数数据
            AccountSecurity security = securityRepository.getByTarget(manager.getId(), Catalog.Manager);
            security.setSalt(salt);
            // 更新随机数
            securityRepository.update(security);
            return new Result(true, "修改密码成功。");
        } catch (Exception e) {
            logger.error("change manager password fail:", e);
            return BocloudResult.FAILED("修改密码失败," + e.getMessage());
        }
    }

    @Override
    public Result resetPwd(Long id, String password, RequestContext requestContext) {
        String path = Manager.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Manager manager = managerRepository.query(id);
            Assert.notNull(manager, "管理员信息不存在！");
            String salt = UUID.randomUUID().toString();
            // 加密
            SHAEncryptor sha = new SHAEncryptor();
            manager.setPassword(sha.encrypt(new AESEncrptor().decrypt(password.trim(), null), salt));
            manager.setMenderId(requestContext.getTarget());
            manager.setLastPwdModifyDate(new Date());
            managerRepository.update(manager);
            // 准备随机数数据
            AccountSecurity managerSecurity = securityRepository.getByTarget(manager.getId(), Catalog.Manager);
            managerSecurity.setSalt(salt);
            // 更新随机数
            securityRepository.update(managerSecurity);
            return new Result(true, "重置密码成功");
        } catch (Exception e) {
            logger.error("reset manager password fail:", e);
            return BocloudResult.FAILED("重置密码失败," + e.getMessage());
        }
    }

    @Override
    @Transactional
    public Result accredit(Long id, List<Long> roleIds, RequestContext requestContext) {
        String path = Manager.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Manager manager = managerRepository.query(id);
            Assert.notNull(manager, "管理员信息不存在！");
            List<ManagerRole> roles = roleIds.stream().map(role -> new ManagerRole(id, role)).collect(Collectors.toList());
            if (!roles.isEmpty()) {
                batchSaveRoles(roles);
            }
            // 更新管理员操作
            manager.setMenderId(requestContext.getTarget());
            managerRepository.update(manager);
            return new Result(true, "管理员角色授权成功");
        } catch (Exception e) {
            logger.error("accredit manager roles fail:", e);
            throw new InternalServerException("管理员角色授权失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 批量保存管理员角色
     *
     * @param roles 角色集合
     * @throws InternalServerException
     */
    @Transactional(rollbackFor = InternalServerException.class)
    public void batchSaveRoles(List<ManagerRole> roles) throws InternalServerException {
        try {
            for (ManagerRole role : roles) {
                managerRoleRepository.save(role);
            }
        } catch (Exception e) {
            throw new InternalServerException("批量插入失败", e, e.getMessage());
        }
    }

    /**
     * 查看管理员详情
     *
     * @param id
     * @return
     */
    @Override
    public BocloudResult<Manager> detail(Long id) {
        try {
            Manager manager = managerRepository.query(id);
            Assert.notNull(manager, "Manager don't exists");
            //获取管理员角色
            List<Role> roles = roleRepository.listByManager(manager.getId());
            if (!roles.isEmpty()) {
                List<String> roleNames = new ArrayList<>();
                for (Role role : roles) {
                    roleNames.add(role.getName());
                }
                manager.setRoleNames(roleNames);
            }
            if (null != manager.getDepartId() && 0L != manager.getDepartId()) {
                Department department = departmentRepository.query(manager.getDepartId());
                if (department != null) {
                    manager.setDepartName(department.getName());
                }
            }
            return new BocloudResult<>(true, manager, "查询管理员详情成功");
        } catch (Exception e) {
            logger.error("query manager detail fail:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    /**
     * 查询管理员列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Override
    public BocloudResult<GridBean<Manager>> list(Pager pager, RequestContext requestContext) {
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            List<Manager> list;
            int total;
            GridBean<Manager> gridBean;
            Long roleId = null;
            sorter.put("gmtCreate", Common.ONE);
            //获取查询参数
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("roleId")) {
                    roleId = Long.valueOf(paramMap.get("roleId").toString());
                    paramMap.remove("roleId");
                }
            }
            total = managerRepository.count(params, roleId);
            if (pager.getSimple()) {
                List<SimpleBean> beans = managerRepository.list(params, sorter, roleId);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = managerRepository.list(pager.getPage(), pager.getRows(), params, sorter, roleId);
                if (!list.isEmpty()) {
                    for (Manager manager : list) {
                        //获取并设置角色名
                        List<Role> roles = roleRepository.listByManager(manager.getId());
                        if (!roles.isEmpty()) {
                            List<String> roleNames = new ArrayList<>();
                            for (Role role : roles) {
                                roleNames.add(role.getName());
                            }
                            manager.setRoleNames(roleNames);
                        }
                    }
                }
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询管理员成功");
        } catch (Exception e) {
            logger.error("list manager fail:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    /**
     * 获取安全密钥
     *
     * @param apiKey
     * @return
     */
    @Override
    public BocloudResult<AccountSecurity> getSecurity(String apiKey) {
        try {
            AccountSecurity security = securityRepository.getByApiKey(apiKey, Catalog.Manager);
            return new BocloudResult<>(true, security, "获取安全密钥成功");
        } catch (Exception e) {
            logger.error("get secKey fail:", e);
            return new BocloudResult<>(false, "获取安全密钥失败");
        }
    }

    /**
     * 校验密码
     *
     * @param id
     * @param password
     * @return
     */
    @Override
    public Result check(Long id, String password) {
        try {
            Manager manager = managerRepository.query(id);
            AccountSecurity security = securityRepository.getByTarget(id, Catalog.Manager);
            Assert.isTrue(!(null == manager || null == security), "原密码验证失败");
            SHAEncryptor sha = new SHAEncryptor();
            String encrypt = sha.encrypt(password, security.getSalt());
            Assert.isTrue(!(null == encrypt || !encrypt.equals(manager.getPassword())), "原密码验证失败");
            return new Result(true, "原密码正确");
        } catch (Exception e) {
            logger.error("old password not correct:", e);
            return new Result(false, e.getMessage());
        }
    }

    /**
     * 检查管理员登录状态
     *
     * @param id
     * @param token
     * @return
     */
    @Override
    public Result checkManager(Long id, String token) {
        try {
            Manager manager = managerRepository.query(id);
            if (null != manager) {
                if (BaseStatus.ABNORMAL.name().equals(manager.getStatus())) {
                    return new Result(false, "该管理员已被冻结，请联系超级管理员进行解冻!");
                }
                String tokenKey = Common.TOKEN + "_" + Catalog.Manager + "_" + id;
                String newTokenCache = this.redisTemplate.opsForValue().get(tokenKey + "_n");
                if (!token.equals(newTokenCache)) {
                    return BocloudResult.FAILED("此管理员已在别处登录!");
                }
                return new Result(true, "");
            } else {
                return new Result(false, "该管理员账号不存在!");
            }
        } catch (Exception e) {
            logger.error("check manager login fail:", e);
            return new Result(false, "检查管理员登录失败!");
        }
    }

    /**
     * 查询管理员的角色
     *
     * @param id             管理员id
     * @param requestContext
     * @return
     */
    @Override
    public BocloudResult<List<Role>> listRoles(Long id, RequestContext requestContext) {
        try {
            List<Role> list = this.roleRepository.listByManager(id);
            List<Role> roles = this.roleRepository.listManagerRole();
            if (null == roles || roles.isEmpty()) {
                return new BocloudResult<>(false, "查询所有角色失败！");
            }
            //构建返回对象，设置check值
            for (Role role : roles) {
                if (null != list && !list.isEmpty()) {
                    for (Role r : list) {
                        //管理员拥有的角色，check设置为true（前端显示用）
                        if (role.getId().equals(r.getId())) {
                            role.setChecked(true);
                        }
                    }
                }
            }
            return new BocloudResult<>(true, roles, "查询成功");
        } catch (Exception e) {
            logger.error("list permission by manager id:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    /**
     * 根据管理员账号冻结管理员基本信息
     *
     * @param account
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result lockByAccount(String account) throws InternalServerException {
        String path;
        boolean result;
        Manager manager;
        try {
            manager = managerRepository.getByAccount(account);
            Assert.notNull(manager, "manager don't exists");
        } catch (Exception e) {
            logger.error("get manager data fail:", e);
            return new Result(false, "获取管理員信息失败");
        }
        path = Manager.class.getSimpleName() + "_" + manager.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            result = managerRepository.lock(manager.getId(), 1L);
            return result ? new Result(true, "冻结管理员成功") : new Result(false, "冻结管理员失败");
        } catch (Exception e) {
            logger.error("freeze manager fail:", e);
            throw new InternalServerException("冻结管理员失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 查询管理员权限树形列表
     *
     * @param context
     * @return
     */
    @Override
    public BocloudResult<List<PermissionBean>> listPermissions(RequestContext context) {
        try {
            return getPermissions(context.getTarget());
        } catch (Exception e) {
            logger.error("freeze manager fail:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    /**
     * 查询所有管理员
     *
     * @return
     */
    @Override
    public BocloudResult<List<Manager>> list() {
        try {
            List<Manager> managers = managerRepository.list();
            return new BocloudResult<>(true, managers, "查询成功");
        } catch (Exception e) {
            logger.error("list all manager error", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    /**
     * 保存管理员登录数据
     *
     * @param log 登录日志
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result save(LoginRecord log) throws InternalServerException {
        String path = LoginRecord.class.getSimpleName() + "_" + log.getAccount();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            loginRecordRepository.save(log);
            return new Result(true, "管理员登录日志记录成功！");
        } catch (Exception e) {
            logger.error("save manager login log fail:", e);
            throw new InternalServerException("管理员登录日志记录失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 管理员登录记录列表展示
     *
     * @param pager
     * @return
     */
    @Override
    public BocloudResult<GridBean<LoginRecord>> list(Pager pager) {
        int total;
        GridBean<LoginRecord> gridBean;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            sorter.put("gmt_create", Common.ONE);
            total = managerRepository.loginTimes(params);
            List<LoginRecord> list = loginRecordRepository.listLoginRecord(pager.getPage(), pager.getRows(), params, sorter, Catalog.Manager);
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query manager login record list fail:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    /**
     * 获取管理员菜单权限
     *
     * @param managerId
     * @return
     */
    private BocloudResult<List<PermissionBean>> getPermissions(Long managerId) {
        try {
            // 查询所有权限(菜单，按钮，api)
            List<Permission> permissions = permissionRepository.getPermissions(managerId);
            // 查询所有权限
            Map<Long, Permission> allPermissions = permissionRepository.list(false).stream().collect(Collectors.toMap(Permission::getId, Function.identity()));
            List<Permission> parents = new ArrayList<>();
            for (Permission permission : permissions) {
                if ("menu".equals(permission.getCategory()) || "view".equals(permission.getCategory())) {
                    //获取父权限
                    this.getParents(allPermissions, parents, permission);
                }
            }
            if (!ListTool.isEmpty(parents)) {
                permissions.addAll(parents);
            }
            //权限去重
            Map<Long, Permission> permissionMap = new HashMap<>(permissions.size(), 0.75F);
            for (Permission permission : permissions) {
                permissionMap.put(permission.getId(), permission);
            }
            permissions = new ArrayList<>(permissionMap.values());
            Map<Long, PermissionBean> permission = new HashMap<>(permissions.size(), 0.75F);
            //重新构建返回权限数据实体，并去重
            for (Permission permit : permissions) {
                if ("menu".equals(permit.getCategory()) || "view".equals(permit.getCategory())) {
                    PermissionBean permissionBean = new PermissionBean(permit.getId(), permit.getName(), permit.getPath(),
                            permit.getIcon(), permit.getParentId(), permit.getPriority(), permit.getRouter(),
                            permit.getComponent(), permit.getRedirect(), permit.getMeta(), permit.getParams(),
                            permit.getCategory(),permit.getPluginsCode(),permit.getPluginsStatus());
                    permission.put(permissionBean.getId(), permissionBean);
                }
            }
            // 菜单权限
            List<PermissionBean> permissionBeans = new ArrayList<>(permission.values());
            List<PermissionBean> resultPermission = new ArrayList<>();
            List<String> plugins = licenseDaemon.content().getPlugins();
            if (plugins==null) {
                // 对权限菜单进行递归排序
                List<PermissionBean> menus = getPermissions(0L, permissionBeans);
                return new BocloudResult<>(true, menus, "获取菜单权限信息成功");
            }else{
                Optional.ofNullable(plugins).orElse(new ArrayList<>()).forEach(plugin -> {
                    Optional.ofNullable(permissionBeans).orElse(new ArrayList<>()).forEach(permissionBean -> {
                        // 获取到插件编码
                        String beanPluginsCode = permissionBean.getPluginsCode();
                        boolean pluginsStatus = permissionBean.getPluginsStatus() != null && permissionBean.getPluginsStatus();
                        // 如果该插件为开启状态，则进行添加操作
                        if (pluginsStatus){
                            String pluginsCode = (beanPluginsCode == null ? "" : beanPluginsCode);
                            if (pluginsCode.equals(plugin)){
                                resultPermission.add(permissionBean);
                            }
                        }
                    });
                });
                // 对权限菜单进行递归排序
                List<PermissionBean> menus = getPermissions(0L, resultPermission);
                return new BocloudResult<>(true, menus, "获取菜单权限信息成功");
            }
        } catch (Exception e) {
            logger.error("get user permission error ", e);
            return new BocloudResult<>(false, "获取权限信息失败:"+e.getMessage());
        }
    }

    /**
     * 返回账号相关登录所需信息
     *
     * @param account
     * @return
     */
    @Override
    public BocloudResult<Map<String, Object>> tokenLogin(String account) {
        try {
            // 判断用户是否存在
            Manager manager;
            manager = managerRepository.getByAccount(account);
            Assert.notNull(manager, "该账号存在问题,校验失败");
            Assert.isTrue(!(manager.getStatus().equals(BaseStatus.ABNORMAL.name())), "该账号用户已被冻结，用户无法登录");
            // 查询随机数
            AccountSecurity security = securityRepository.getByTarget(manager.getId(), Catalog.Manager);
            Assert.notNull(security, "该账号存在问题,校验失败");
            Map<String, Object> map = MapTools.simpleMap("manager", manager);
            map.put(Common.API_KEY, security.getApiKey());
            map.put(Common.SEC_KEY, security.getSecKey());
            map.put("accountCategory", Catalog.Manager.name());
            return new BocloudResult<>(true, map, "用户跳转成功");
        } catch (Exception e) {
            logger.error("Get manager fail by account:", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }


    /**
     * 根据管理员账号查询管理员信息
     *
     * @param account 管理员账号
     * @return
     */
    @Override
    public Result exist(String account) {
        try {
            Manager manager = managerRepository.getByAccount(account);
            return manager == null ? new Result(true, "账号可以使用") : new Result(false, "账号已经存在");
        } catch (Exception e) {
            logger.error("get account by account error,", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }
}
