package com.bocloud.sms.service;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.NameRule;
import com.bocloud.sms.entity.Project;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.NameRuleService;
import com.bocloud.sms.repository.*;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.security.SecureRandom;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 命名规则service实现
 *
 * @author wangyu
 * @version 3.0.0-SNAPSHOT
 * @since 2018/7/2
 */
@Service("nameRuleService")
public class NameRuleServiceImpl implements NameRuleService {

    private static final Logger logger = LoggerFactory.getLogger(NameRuleServiceImpl.class);

    private final NameRuleRepository nameRuleRepository;
    private final LockFactory lockFactory;
    private final TenantRepository tenantRepository;
    private final UserRepository userRepository;
    private final ProjectRepository projectRepository;
    private final BusinessRepository businessRepository;

    public NameRuleServiceImpl(NameRuleRepository nameRuleRepository, LockFactory lockFactory, TenantRepository tenantRepository, UserRepository userRepository, ProjectRepository projectRepository, BusinessRepository businessRepository) {
        this.nameRuleRepository = nameRuleRepository;
        this.lockFactory = lockFactory;
        this.tenantRepository = tenantRepository;
        this.userRepository = userRepository;
        this.projectRepository = projectRepository;
        this.businessRepository = businessRepository;
    }

    @Override
    public BocloudResult<GridBean<NameRule>> list(Pager pager, RequestContext context) {
        List<NameRule> list;
        int total;
        GridBean<NameRule> gridBean;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
            int page = Optional.ofNullable(pager.getPage()).orElse(1);
            int rows = Optional.ofNullable(pager.getRows()).orElse(10);

            params.add(new Param(MapTools.simpleMap(Common.TENANTID, 0L), Sign.EQ));
            total = nameRuleRepository.count(params);
            if (simple) {
                List<NameRule> beans = this.nameRuleRepository.list(params);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = nameRuleRepository.list(page, rows, params, sorter);
                gridBean = new GridBean<>(page, rows, total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list name rule failure:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<NameRule> detail(Long id) {
        try {
            NameRule nameRule = nameRuleRepository.query(id);
            return new BocloudResult<>(true, nameRule, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<NameRule> create(NameRule nameRule, RequestContext context) {
        try {
            if (nameRuleRepository.exists(nameRule.getName(), context.getTenant())) {
                return new BocloudResult<>(false, "名称已存在");
            }
            if (nameRuleRepository.exists(nameRule.getHasTenant(), nameRule.getHasUser(), nameRule.getHasBusiness(),
                    nameRule.getHasProject(), nameRule.getCustomize(), nameRule.getSuffixType(), context.getTenant())) {
                return new BocloudResult<>(false, "存在重复的命名规则");
            }
            if (null == nameRule.getCurrentNum()) {
                nameRule.setCurrentNum(0L);
            }
            nameRule.setTenantId(context.getTenant());
            nameRuleRepository.save(nameRule);
            return new BocloudResult<>(true, nameRule, "添加成功");
        } catch (Exception e) {
            logger.error("Create name rule error:", e);
            return new BocloudResult<>(false, "添加失败");
        }
    }

    @Override
    public Result modify(NameRule nameRule, RequestContext context) {
        String path = NameRule.class.getSimpleName() + "_" + nameRule.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NameRule oldRule = this.nameRuleRepository.query(nameRule.getId());
            if (null == oldRule) {
                logger.warn("Name rule does not exist!");
                return new Result(false, "数据不存在");
            }
            if (!nameRule.getName().equals(oldRule.getName())) {
                if (nameRuleRepository.exists(nameRule.getName(), context.getTenant())) {
                    return new Result(false, "名称已存在");
                }
            }
            oldRule.setName(nameRule.getName());
            if (nameRule.getHasTenant().equals(oldRule.getHasTenant())
                    || nameRule.getHasUser().equals(oldRule.getHasUser())
                    || nameRule.getHasBusiness().equals(oldRule.getHasBusiness())
                    || nameRule.getHasProject().equals(oldRule.getHasProject())
                    || !nameRule.getCustomize().equals(oldRule.getCustomize())
                    || !nameRule.getSuffixType().equals(oldRule.getSuffixType())) {
                //去掉重复校验
//                if (nameRuleRepository.exists(nameRule.getHasTenant(), nameRule.getHasUser(), nameRule.getHasBusiness(),
//                        nameRule.getHasProject(), nameRule.getCustomize(), nameRule.getSuffixType(),
//                        context.getTenant())) {
//                    return new Result(false, "存在重复的命名规则");
//                }
                oldRule.setSuffixType(nameRule.getSuffixType());
                oldRule.setSuffixLength(nameRule.getSuffixLength());
                oldRule.setHasTenant(nameRule.getHasTenant());
                oldRule.setHasUser(nameRule.getHasUser());
                oldRule.setHasBusiness(nameRule.getHasBusiness());
                oldRule.setHasProject(nameRule.getHasProject());
                oldRule.setCustomize(nameRule.getCustomize());
            }
            this.nameRuleRepository.update(oldRule);
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Modify name rule error:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public Result remove(Long id) {
        String path = NameRule.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NameRule nameRule = this.nameRuleRepository.query(id);
            if (null == nameRule) {
                logger.warn("Name rule does not exist!");
                return new Result(false, "数据不存在");
            }
            this.nameRuleRepository.remove(id);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove name rule error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public BocloudResult<String> generate(Long id, Long tenantId, Long userId, Long projectId, RequestContext context) {
        try {
            NameRule nameRule = this.nameRuleRepository.query(id);
            if (null == nameRule) {
                logger.warn("Name rule does not exist!");
                return new BocloudResult<>(false, "命名规则不存在");
            }
            String catalog = context.getCatalog().name();
            String customize = nameRule.getCustomize();
            String name = "";
            if (null != tenantId && 0L != tenantId && null != nameRule.getHasTenant() && nameRule.getHasTenant()) {
                Tenant tenant = tenantRepository.query(tenantId);
                if (null == tenant) {
                    return new BocloudResult<>(false, "命名生成失败，租户不存在");
                }
                name += tenant.getAccount();
            }
            if (null != userId && 0L != userId && null != nameRule.getHasUser() && nameRule.getHasUser()) {
                if (catalog.equalsIgnoreCase(RequestContext.Catalog.User.name())) {
                    User manager = userRepository.query(userId);
                    if (null == manager) {
                        return new BocloudResult<>(false, "命名生成失败，用户不存在");
                    }
                    if (StringUtils.isEmpty(name)) {
                        name += manager.getAccount();
                    } else {
                        name += "_" + manager.getAccount();
                    }
                } else if (catalog.equalsIgnoreCase(RequestContext.Catalog.Tenant.name())) {
                    Tenant tenant = tenantRepository.query(tenantId);
                    if (null == tenant) {
                        return new BocloudResult<>(false, "命名生成失败，租户不存在");
                    }
                    name += tenant.getAccount();
                } else {
                    return new BocloudResult<>(false, "未知用户角色");
                }
            }
            if (null != projectId && 0L != projectId) {
                Project project = projectRepository.query(projectId);
                if (null == project) {
                    return new BocloudResult<>(false, "命名生成失败，项目不存在");
                }
                //暂不需要业务
//                if (null != nameRule.getHasBusiness() && nameRule.getHasBusiness()) {
//                    Business business = businessRepository.query(project.getBusinessId());
//                    if (null == business) {
//                        return new BocloudResult<>(false, "命名生成失败，业务不存在");
//                    }
//                    if (StringUtils.isEmpty(name)) {
//                        name += business.getCode();
//                    } else {
//                        name += "_" + business.getCode();
//                    }
//                }
                if (null != nameRule.getHasProject() && nameRule.getHasProject()) {
                    if (StringUtils.isEmpty(name)) {
                        name += project.getCode();
                    } else {
                        name += "_" + project.getCode();
                    }
                }
            }
            if (!StringUtils.isEmpty(customize)) {
                if (StringUtils.isEmpty(name)) {
                    name += customize;
                } else {
                    name += "_" + customize;
                }
            }
            // 后缀
            String suffixType = nameRule.getSuffixType();
            String suffix = null;
            if (NameRule.SuffixType.Millis.name().equalsIgnoreCase(suffixType)) {
                suffix = String.valueOf(System.currentTimeMillis());
            } else if (NameRule.SuffixType.RandomString.name().equalsIgnoreCase(suffixType)) {
                Long suffixLength = nameRule.getSuffixLength();
                if (null == suffixLength || 0L == suffixLength) {
                    suffixLength = 10L;
                }
                suffix = getRandomString(suffixLength.intValue());
            } else if (NameRule.SuffixType.Number.name().equalsIgnoreCase(suffixType)) {
                Long nextNum = null == nameRule.getCurrentNum() ? 1L : nameRule.getCurrentNum();
                suffix = String.valueOf(nextNum);
                // 更新当前编号
                nameRule.setCurrentNum(nameRule.getCurrentNum() + 1L);
                String path = NameRule.class.getSimpleName() + "_" + id;
                try (BocloudACLock lock = lockFactory.getACLock(path)) {
                    Assert.isTrue(lock.acquire(20, TimeUnit.SECONDS), "请求超时");
                    nameRuleRepository.update(nameRule);
                } catch (Exception e) {
                    logger.error("Remove name rule error:", e);
                    return new BocloudResult<>(false, "生成失败");
                }
            }
            return new BocloudResult<>(true, name + "_" + suffix, "生成成功");
        } catch (Exception e) {
            logger.error("Remove name rule error:", e);
            return new BocloudResult<>(false, "生成失败");
        }
    }

    // 生成指定length的随机字符串（A-Z，a-z，0-9）
    public static String getRandomString(int length) {
        String str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        Random random = new SecureRandom();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(str.length());
            sb.append(str.charAt(number));
        }
        return sb.toString();
    }

}
