package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BaseStatus;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.RoleService;
import com.bocloud.sms.model.ApiInfo;
import com.bocloud.sms.model.CloudServiceBean;
import com.bocloud.sms.model.RoleModel;
import com.bocloud.sms.repository.*;
import com.bocloud.sms.utils.PermissionComparator;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 角色抽象Service接口实现类
 *
 * @author tyl
 */
@Service("roleService")
public class RoleServiceImpl implements RoleService {

    private final Logger logger = LoggerFactory.getLogger(RoleServiceImpl.class);
    private final RoleRepository roleRepository;
    private final PermissionRepository permissionRepository;
    private final TenantRepository tenantRepository;
    private final UserRepository userRepository;
    private final StringRedisTemplate redisTemplate;
    private final LockFactory lockFactory;
    private final CloudServiceBeanRepository cloudServiceBeanRepository;
    private final RolePermissionRepository rolePermissionRepository;
    private final ServiceRoleRepository serviceRoleRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ApiPermissionRepository apiPermissionRepository;

    public RoleServiceImpl(RoleRepository roleRepository, PermissionRepository permissionRepository, TenantRepository tenantRepository, UserRepository userRepository, StringRedisTemplate redisTemplate, LockFactory lockFactory, CloudServiceBeanRepository cloudServiceBeanRepository, RolePermissionRepository rolePermissionRepository, ServiceRoleRepository serviceRoleRepository, CloudServiceRepository cloudServiceRepository, ApiPermissionRepository apiPermissionRepository) {
        this.roleRepository = roleRepository;
        this.permissionRepository = permissionRepository;
        this.tenantRepository = tenantRepository;
        this.userRepository = userRepository;
        this.redisTemplate = redisTemplate;
        this.lockFactory = lockFactory;
        this.cloudServiceBeanRepository = cloudServiceBeanRepository;
        this.rolePermissionRepository = rolePermissionRepository;
        this.serviceRoleRepository = serviceRoleRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.apiPermissionRepository = apiPermissionRepository;
    }

    /**
     * 添加角色
     *
     * @param roleModel
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result create(RoleModel roleModel, RequestContext requestContext) throws InternalServerException {
        try {
            Role role = new Role();
            BeanUtils.copyProperties(roleModel, role);
            Assert.notNull(role, "角色数据不能为空。");
            role.setCreatorId(requestContext.getTarget());
            // 判断是否是租户请求添加角色
            if (null != requestContext.getTenant() && requestContext.getTenant() > 0) {
                role.setTenantId(requestContext.getTenant());
            } else {
                role.setTenantId(0L);
            }
            Result exists = exists(role.getName(), role.getTenantId());
            Assert.isTrue(exists.isFailed(), "角色名称已存在。");
            role.setStatus(BaseStatus.NORMAL.name());
            roleRepository.save(role);
            return new Result(true, "添加角色成功");
        } catch (Exception e) {
            logger.error("create role fail:", e);
            throw new InternalServerException("添加角色失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 修改角色
     *
     * @param roleModel      角色信息
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result modify(Long id, RoleModel roleModel, RequestContext requestContext) throws InternalServerException {
        String path = Role.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Role role = roleRepository.query(id);
            Assert.notNull(role, "角色数据不存在。");
            if (!roleModel.getName().equals(role.getName())) {
                //判断修改的角色名是否存在
                Result exists = exists(roleModel.getName(), role.getTenantId());
                Assert.isTrue(exists.isFailed(), "角色名称已存在");
            }
            BeanUtils.copyProperties(roleModel, role);
            role.setMenderId(requestContext.getTarget());
            boolean result = roleRepository.update(role);
            return result ? new Result(true, "修改角色成功") : new Result(false, "修改角色失败");
        } catch (Exception e) {
            logger.error("modify role fail:", e);
            throw new InternalServerException("修改角色失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 删除角色
     *
     * @param id             角色id
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result remove(Long id, RequestContext requestContext) throws InternalServerException {
        String path = Role.class.getSimpleName() + "_" + id;
        boolean result;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Role role = roleRepository.query(id);
            Assert.notNull(role, "角色数据不存在。");
            if (id.equals(1L)) {
                return new Result(false, "系统初始化角色不能删除!");
            }
            // 删除角色和权限、用户、管理员的对应关系
            // 先解除角色权限绑定 再解除角色绑定
            this.rolePermissionRepository.unBindPermission(id);
            this.roleRepository.unBindManager(id);
            this.roleRepository.unBindUser(id);
            result = roleRepository.delete(id, requestContext.getTarget());
        } catch (Exception e) {
            logger.error("remove role fail:", e);
            throw new InternalServerException("删除角色失败," + e.getMessage(), e, e.getMessage());
        }
        return result ? new Result(true, "删除角色成功") : new Result(false, "删除角色失败");
    }

    /**
     * 查询角色详情
     *
     * @param id 角色id
     * @return
     */
    @Override
    public BocloudResult<Role> detail(Long id) {
        try {
            Role role = roleRepository.query(id);
            return new BocloudResult<>(true, role, "查询角色详情成功");
        } catch (Exception e) {
            logger.error("query user detail fail:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 授权角色权限
     *
     * @param id             角色id
     * @param ids            自服务服务id 管理端权限id
     * @param requestContext
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class/*, propagation = Propagation.SUPPORTS*/)
    public Result accredit(Long id, List<Long> ids, RequestContext requestContext) {
        String path = Role.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            //管理端授权角色，角色和权限直接关联
            if (RequestContext.Catalog.Manager.equals(requestContext.getCatalog())) {
                Role role = roleRepository.query(id);
                Assert.notNull(role, "role data don't exist");
                // 先解绑角色下的所有权限
                this.rolePermissionRepository.unBindPermission(id);
                //生成角色权限对象数组
                List<RolePermission> rolePermissions = ids.stream().map(permissionId -> new RolePermission(null, id, permissionId)).collect(Collectors.toList());
                //重新授权角色权限
                if (!rolePermissions.isEmpty()) {
                    batchSavePermissions(rolePermissions);
                }
                // 更新用户操作
                role.setMenderId(requestContext.getTarget());
                roleRepository.update(role);
            } else {
                //自服务端授权角色，角色和服务关联
                Role role = roleRepository.query(id);
                Assert.notNull(role, "role data don't exist");

                //构建服务角色关联
                List<ServiceRole> serviceRoles = ids.stream().map(serviceId -> new ServiceRole(null, id, serviceId)).collect(Collectors.toList());
                roleRepository.unBindService(id);
                if (!serviceRoles.isEmpty()) {
                    for (ServiceRole serviceRole : serviceRoles) {
                        serviceRoleRepository.save(serviceRole);
                    }
                    // roleRepository.bindServices(serviceRoles);
                }
                // 更新用户操作
                role.setMenderId(requestContext.getTarget());
                roleRepository.update(role);
            }
            return new Result(true, "授权角色权限成功");
        } catch (Exception e) {
            logger.error("accredit role authority fail:", e);
            throw new InternalServerException("授权角色权限失败," + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 保存角色权限数据
     *
     * @param permissions 权限集合
     * @throws InternalServerException
     */
    @Transactional(rollbackFor = InternalServerException.class)
    public void batchSavePermissions(List<RolePermission> permissions) throws InternalServerException {
        try {
            rolePermissionRepository.bindPermissions(permissions);
        } catch (Exception e) {
            throw new InternalServerException("批量插入失败", e, e.getMessage());
        }
    }

    /**
     * 查询角色列表
     *
     * @param pager          分页对象
     * @param requestContext 请求对象
     * @return
     */
    @Override
    public BocloudResult<GridBean<Role>> list(Pager pager, RequestContext requestContext) {
        try {
            List<Role> list;
            int total;
            GridBean<Role> gridBean;
            List<SimpleBean> beans;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            sorter.put("gmtCreate", Common.ONE);
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            //判断是否是查询租户（自服务端）角色，并把角色id添加进查询参数中
            if (null != requestContext.getTenant() && requestContext.getTenant() > 0) {
                params.add(new Param(MapTools.simpleMap(Common.TENANTID, requestContext.getTenant()), Sign.EQ));
            } else {
                params.add(new Param(MapTools.simpleMap(Common.TENANTID, 0), Sign.EQ));
            }
            total = roleRepository.count(params);
            if (pager.getSimple()) {
                beans = roleRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = roleRepository.list(pager.getPage(), pager.getRows(), params, sorter);
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询角色成功");
        } catch (Exception e) {
            logger.error("list role fail:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 获取角色权限
     *
     * @param id             角色id
     * @param menu           是否是菜单权限
     * @param tenant         是否是租户权限（自服务端）
     * @param requestContext
     * @return
     */
    @Override
    public BocloudResult<List<CloudServiceBean>> listPermissions(Long id, boolean menu, boolean tenant, RequestContext requestContext) {
        try {
            //自服务端查询，返回角色绑定服务关系
            if (RequestContext.Catalog.Tenant.equals(requestContext.getCatalog())) {
                //获取已经授权的服务
                List<CloudService> roleServices = cloudServiceRepository.listRoleServices(id);
                List<CloudServiceBean> cloudServiceBeans = cloudServiceBeanRepository.getFavoriteServices(requestContext.getTenant(), RequestContext.Catalog.Tenant.name());
                for (CloudServiceBean cloudServiceBean : cloudServiceBeans) {
                    for (CloudService serviceCode : roleServices) {
                        if (serviceCode.getCode().equals(cloudServiceBean.getCode())) {
                            cloudServiceBean.setSubscribe(true);
                        }
                    }
                }
                //排序
                cloudServiceBeans.sort((first, second) -> null != first && second != null ? first.getPriority().compareTo(second.getPriority()) : 0);
                return new BocloudResult<>(true, cloudServiceBeans, "查询成功");
            } else {
                // 非租户菜单权限
                if (menu) {
                    //获取角色权限
                    List<Permission> rolePermissions = permissionRepository.listByRole(id);
                    // 查询非租户菜单权限
                    List<Permission> menus = permissionRepository.list(false);
                    for (Permission permits : menus) {
                        for (Permission permission : rolePermissions) {
                            //角色如果有这个权限，checked设置为true
                            if (permits.getId().equals(permission.getId())) {
                                permits.setChecked(true);
                            }
                        }
                    }
                    return new BocloudResult(true, getPermissions(0L, menus), "查询成功");
                } else {
                    //获取按钮权限
                    List<Permission> roleButtons = permissionRepository.listByCategory(id, "button");
                    List<Permission> buttons = permissionRepository.list(false, "button", null);
                    for (Permission button : buttons) {
                        for (Permission roleButton : roleButtons) {
                            if (button.getId().equals(roleButton.getId())) {
                                button.setChecked(true);
                            }
                        }
                    }
                    //获取api权限
                    List<Permission> roleApis = permissionRepository.listByCategory(id, "api");
                    List<Permission> apis = permissionRepository.list(false, "api", null);
                    for (Permission roleApi : roleApis) {
                        for (Permission api : apis) {
                            if (roleApi.getId().equals(api.getId())) {
                                api.setChecked(true);
                            }
                        }
                    }
                    HashMap<String, Object> permissions = new HashMap<>(2);
                    permissions.put("button", buttons);
                    permissions.put("api", apis);
                    return new BocloudResult(true, permissions, "查询成功！");
                }
            }
        } catch (Exception e) {
            logger.error("list by userId failed", e);
            return new BocloudResult<>(false, "查询失败!");
        }
    }

    /**
     * 对权限进行递归排序
     *
     * @param pid             父权限id
     * @param permissionBeans 需要排序的目标权限集合
     * @return
     */
    private List<Permission> getPermissions(Long pid, List<Permission> permissionBeans) {
        //创建集合用于接收返回递归后的权限数据
        List<Permission> newPermissions = Lists.newArrayList();
        for (Permission permission : permissionBeans) {
            //判断是否权限是否有父权限id
            if (null != permission.getParentId() && permission.getParentId().equals(pid)) {
                //有父权限id，则进入递归获取其子权限
                List<Permission> children = getPermissions(permission.getId(), permissionBeans);
                if (!ListTool.isEmpty(children)) {
                    //排序后设置子权限
                    children.sort(new PermissionComparator());
                    permission.setChilds(children);
                } else {
                    permission.setChilds(null);
                }
                newPermissions.add(permission);
            }
        }
        newPermissions.sort(new PermissionComparator());
        return newPermissions;
    }

    /**
     * 查询父菜单下面所有按钮或api权限
     *
     * @param permissions
     * @param category
     * @param parentId
     * @throws Exception
     */
    @SuppressWarnings("unused")
    private void findButtonOrApi(List<Permission> permissions, String category, Long parentId) throws Exception {
        List<Permission> permits = permissionRepository.listByCategory(category, parentId);
        if (!ListTool.isEmpty(permits)) {
            permissions.addAll(permits);
        }
        List<Permission> children = permissionRepository.list(parentId, false);
        for (Permission child : children) {
            findButtonOrApi(permissions, category, child.getId());
        }
    }

    /**
     * 查询角色名称是否存在
     *
     * @param name     角色名
     * @param tenantId 租户id
     * @return
     */
    @Override
    public Result exists(String name, Long tenantId) {
        try {
            Role role = roleRepository.exists(name, tenantId);
            Assert.notNull(role, "角色不存在");
            return new Result(true, "存在!");
        } catch (Exception e) {
            logger.error("get role failure:", e);
            return new Result(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult<List<Long>> roleIds(RequestContext requestContext) {
        try {
            List<Long> roles;
            if (requestContext.getCatalog() == RequestContext.Catalog.Manager) {
                roles = roleRepository.listByManager(requestContext.getTarget()).stream().map(Role::getId).collect(Collectors.toList());
            } else {
                roles = roleRepository.getTenantApiRole().stream().map(Role::getId).collect(Collectors.toList());
            }
            return new BocloudResult<>(true, roles, "获取角色信息成功");
        } catch (Exception e) {
            logger.error("get account role info error,", e);
            return new BocloudResult<>(false, "获取角色信息失败:" + e.getMessage());
        }
    }

    @Override
    public Result accreditRole(Long id, List<ApiInfo> apiList) {
        String path = Role.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            apiList = Optional.ofNullable(apiList).orElse(new ArrayList<>());
            Map<String, List<ApiInfo>> apiMap = apiList.stream().collect(Collectors.groupingBy(ApiInfo::getService));
            HashMap<String, List<String>> patternsMap = new HashMap<>(8);
            List<ApiPermission> patternsList = new ArrayList<>();
            for (Map.Entry<String, List<ApiInfo>> entry : apiMap.entrySet()) {
                String service = entry.getKey();
                List<ApiInfo> list = entry.getValue();
                List<String> patterns = list.stream().map(ApiInfo::getPattern).collect(Collectors.toList());
                List<ApiPermission> apiPermissions = list.stream()
                        .map(map -> new ApiPermission(null, id, Base64.getEncoder().encodeToString(map.getPattern().getBytes(StandardCharsets.UTF_8)), map.getOperation(), service))
                        .collect(Collectors.toList());
                patternsList.addAll(apiPermissions);
                patternsMap.put(service, patterns);
            }
            Map<String, String> entryMap = new HashMap<>(8);
            for (Map.Entry<String, List<String>> entry : patternsMap.entrySet()) {
                entryMap.put(entry.getKey(), JSONObject.toJSONString(entry.getValue()));
            }
            redisTemplate.opsForHash().delete("api_permission_" + id, "ams", "bms", "cms", "dms", "fms", "ims", "nms", "pms", "rms", "sms", "cop", "cos", "mcs", "gws", "lms", "pontus", "worker");
            redisTemplate.opsForHash().putAll("api_permission_" + id, entryMap);

            ArrayList<Long> list = Lists.newArrayList();
            list.add(id);
            List<ApiPermission> apiPermissions = apiPermissionRepository.listRoleApiPermission(list);
            // 如果有就删除
            if (!apiPermissions.isEmpty()) {
                roleRepository.batchRemoveApiPermission(id);
            }
            if (!patternsList.isEmpty()) {
                apiPermissionRepository.batchSave(patternsList, patternsList.size());
            }
            return new Result(true, "授权成功");
        } catch (Exception e) {
            logger.error("accredit api permission to role error", e);
            return BocloudResult.FAILED("授权失败:" + e.getMessage());
        }
    }


    @Override
    public BocloudResult<JSONArray> getApiPermission(Long id, String tags) {
        try {
            Map<String, List<String>> roleApiMap = new HashMap<>(8);
            Map<Object, Object> roleEntries = redisTemplate.opsForHash().entries("api_permission_" + id);
            for (Map.Entry<Object, Object> entry : roleEntries.entrySet()) {
                String service = (String) entry.getKey();
                List<String> patterns = JSONArray.parseArray(entry.getValue().toString(), String.class);
                roleApiMap.put(service, patterns);
            }
            Map<String, List<ApiInfo>> apiMap = new HashMap<>(8);
            Map<Object, Object> apiEntries = redisTemplate.opsForHash().entries("api_list");
            for (Map.Entry<Object, Object> entry : apiEntries.entrySet()) {
                String service = (String) entry.getKey();
                List<ApiInfo> apiInfos = JSONArray.parseArray(entry.getValue().toString(), ApiInfo.class);
                apiMap.put(service, apiInfos);
            }
            JSONArray result = new JSONArray();
            for (Map.Entry<String, List<ApiInfo>> entry : apiMap.entrySet()) {
                JSONObject serviceApi = new JSONObject();
                String service = entry.getKey();
                String serviceName = getServiceName(service);
                serviceApi.put("name", serviceName);
                serviceApi.put("code", service);
                Map<String, List<ApiInfo>> apiInfoMap = entry.getValue().stream().collect(Collectors.groupingBy(ApiInfo::getTarget));
                JSONArray apiResult = new JSONArray();
                for (Map.Entry<String, List<ApiInfo>> infoEntry : apiInfoMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    String targetName = infoEntry.getKey();
                    List<ApiInfo> value = infoEntry.getValue();
                    ListIterator<ApiInfo> iterator = value.listIterator();
                    while (iterator.hasNext()) {
                        ApiInfo apiInfo = iterator.next();
                        List<String> roleApis = Optional.ofNullable(roleApiMap.get(service)).orElse(new ArrayList<>());
                        if (StringUtils.hasText(tags)) {
                            if (!Arrays.asList(apiInfo.getTags()).contains(tags)) {
                                iterator.remove();
                            }
                        }
                        apiInfo.setCheck(roleApis.contains(apiInfo.getPattern()));
                    }
                    jsonObject.put("name", targetName);
                    jsonObject.put("value", value);
                    apiResult.add(jsonObject);
                }
                serviceApi.put("value", apiResult);
                result.add(serviceApi);
            }
            return new BocloudResult<>(true, result, "查询角色API权限成功");
        } catch (Exception e) {
            logger.error("get role api permission error,", e);
            return new BocloudResult<>(false, "查询角色API权限失败:" + e.getMessage());
        }
    }

    private String getServiceName(String moduleName) {
        if (moduleName != null) {

            if (moduleName.equals(BoCloudService.IMS.getName())) {
               return "IAAS资源管理服务";
            }
            if (moduleName.equals(BoCloudService.PMS.getName())) {
                return "PAAS资源管理服务";
            }
            if (moduleName.equals(BoCloudService.NMS.getName())) {
               return "网络资源管理服务";
            }
            if (moduleName.equals(BoCloudService.DMS.getName())) {
                return "存储资源管理服务";
            }
            if (moduleName.equals(BoCloudService.SMS.getName())) {
                return  "系统服务";
            }
            if (moduleName.equals(BoCloudService.CMS.getName())) {
               return  "监控服务";
            }
            if (moduleName.equals(BoCloudService.COP.getName())) {
                return "运维服务";
            }
            if (moduleName.equals(BoCloudService.COS.getName())) {
                return  "运营服务";
            }
            if (moduleName.equals(BoCloudService.AMS.getName())) {
               return  "资产服务";
            }
            if (moduleName.equals(BoCloudService.BMS.getName())) {
                return  "裸金属服务";
            }
            if (moduleName.equals(BoCloudService.FMS.getName())) {
                return  "流程服务";
            }
            if (moduleName.equals(BoCloudService.RMS.getName())) {
                return  "报表服务";
            }
        }
        return moduleName;
    }

    @Override
    public BocloudResult<GridBean<User>> listUsersByRoleId(Pager pager, Long id) {
        try {
            List<SimpleBean> beans;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            sorter.put("gmtCreate", Common.ONE);
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            int total = userRepository.count(params, id);
            GridBean<User> gridBean;
            if (pager.getSimple()) {
                beans = userRepository.list(params, sorter, id);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                List<User> list = userRepository.list(pager.getPage(), pager.getRows(), params, sorter, id);
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list users by role id error,", e);
            return new BocloudResult<>(false, "根据角色查询用户失败:" + e.getMessage());
        }
    }
}
