package com.bocloud.sms.service;

import com.bocloud.common.enums.BaseStatus;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.sms.entity.SystemTask;
import com.bocloud.sms.interfaces.SystemTaskService;
import com.bocloud.sms.repository.SystemTaskRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.time.*;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 平台任务业务层
 *
 * @author tyl
 * @since 2021/4/1 17:31
 */
@Service("systemTaskServiceImpl")
public class SystemTaskServiceImpl implements SystemTaskService {
    private final SystemTaskRepository systemTaskRepository;
    private final LockFactory lockFactory;
    private final Logger logger = LoggerFactory.getLogger(SystemTaskServiceImpl.class);


    public SystemTaskServiceImpl(SystemTaskRepository systemTaskRepository, LockFactory lockFactory) {
        this.systemTaskRepository = systemTaskRepository;
        this.lockFactory = lockFactory;
    }

    @Override
    public BocloudResult<GridBean<SystemTask>> list(Pager pager) {
        try {
            GridBean<SystemTask> gridBean;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            sorter.put("gmtCreate", Common.ONE);
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            int total = systemTaskRepository.count(params);
            if (pager.getSimple()) {
                List<SimpleBean> beans = systemTaskRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                List<SystemTask> list = systemTaskRepository.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter());
                list.forEach(task->task.setCost(timeInterval(task.getGmtBegin(),task.getGmtEnd())));
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询平台任务成功");
        } catch (Exception e) {
            logger.error("list system task fail:", e);
            return new BocloudResult<>(false, "查询失败:"+e.getMessage());
        }
    }

    @Override
    public Result create(SystemTask systemTask, RequestContext context) {
        try {
            Assert.notNull(systemTask, "平台任务数据不能为空。");
            systemTask.setCreatorId(context.getTarget());
            SystemTask exist = systemTaskRepository.queryByName(systemTask.getName());
            Assert.isNull(exist, "任务名称已存在。");
            systemTask.setStatus(BaseStatus.NORMAL.name());
            systemTask.setDeleted(false);
            systemTask.setGmtCreate(new Date());
            systemTaskRepository.save(systemTask);
            return new Result(true, "添加平台任务成功");
        } catch (Exception e) {
            logger.error("create system task fail:", e);
            return new Result(false, "添加平台任务失败：" + e.getMessage());
        }
    }

    @Override
    public Result remove(Long id, RequestContext context) {
        String path = SystemTask.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SystemTask systemTask = systemTaskRepository.detail(id);
            Assert.notNull(systemTask, "平台任务数据不存在。");
            systemTaskRepository.delete(id, context.getTarget());
            return new Result(true, "删除平台任务成功");
        } catch (Exception e) {
            logger.error("remove system task fail:", e);
            return new Result(false, "删除平台任务失败:" + e.getMessage());
        }
    }

    @Override
    public Result modify(Long id, SystemTask systemTask, RequestContext context) {
        String path = SystemTask.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SystemTask oldSystemTask = systemTaskRepository.detail(id);
            Assert.notNull(oldSystemTask, "平台任务信息不存在");
            if (!systemTask.getName().equals(oldSystemTask.getName())) {
                SystemTask exist = systemTaskRepository.queryByName(systemTask.getName());
                Assert.isNull(exist, "任务名称已存在。");
                oldSystemTask.setName(systemTask.getName());
            }
            oldSystemTask.setContent(systemTask.getContent());
            oldSystemTask.setGmtModify(new Date());
            oldSystemTask.setMenderId(context.getTarget());
            systemTaskRepository.update(oldSystemTask);
            return new Result(true,  "修改平台任务成功");
        } catch (Exception e) {
            logger.error("modify system task error,", e);
            return BocloudResult.FAILED("修改平台任务失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<SystemTask> detail(Long id) {
        try {
            SystemTask systemTask = systemTaskRepository.detail(id);
            return new BocloudResult<>(true, systemTask, "查询成功");
        } catch (Exception e) {
            logger.error("get system task detail error,", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private  String timeInterval(Date begin, Date end){
        LocalDateTime beginData = begin.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        LocalDateTime endData = end.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        Period period = Period.between(beginData.toLocalDate(), endData.toLocalDate());
        StringBuilder compactTime = new StringBuilder();
        if (period.getYears() > 0) {
            compactTime.append(period.getYears()).append("年");
        }
        if (period.getMonths() > 0) {
            compactTime.append(period.getMonths()).append("月");
        }
        if (period.getDays() > 0) {
            compactTime.append(period.getDays()).append("天");
        }
        Duration duration = Duration.between(beginData, endData);
        if (duration.toHours()%24>0){
            compactTime.append(duration.toHours()%24).append("小时");
        }
        if (duration.toMinutes()%60>0){
            compactTime.append(duration.toMinutes()%60).append("分");
        }
        if (duration.toMillis()/1000%60>0){
            compactTime.append(duration.toMillis()/1000%60).append("秒");
        }
        return compactTime.toString();
    }
}
