package com.bocloud.sms.service.upgrade.tasks;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * cms数据迁移任务
 *
 * 新增加的字段留空，直接迁移5.0数据即可
 *
 * @author zdy
 * @date 2021-07-14
 */
@Component
public class CmsTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context
     *            spring上下文
     * @param userTask
     *            依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public CmsTask(ApplicationContext context, UserTask userTask, UnchangedTask unchangedTask) {
        this.context = context;
        this.dependOn.add(userTask);
        this.dependOn.add(unchangedTask);
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "cms数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "cms", COLUMNS,
            percent -> this.percent = Math.floorDiv(percent, 2));
        // 更新cms表中expired_time的数据：object_relation表中target==cms.id且catalog==cms的数据
        String updateSql = "update cms k left join "
            + " (select o.expired_time, o.target from object_relation o left join cms k on k.id = o.target and o.catalog = 'cms') ob "
            + "on k.id = ob.target set k.expired_time = ob.expired_time where 1=1";
        targetJdbcTemplate.update(updateSql, new HashMap<>());
        this.percent = 50;
        // 处理creator_id、mender_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cms", "creator_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cms", "mender_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "cms", "owner_id", id.getKey(),
                id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

    // 5.0字段
    private static final String[] COLUMNS =
        new String[] {"id", "name", "gmt_create", "remark", "gmt_modify", "creator_id", "owner_id", "mender_id",
            "status", "is_deleted", "props", "tenant_id", "region", "uuid", "instance_id", "type", "category",
            "version", "sku_id", "network_id", "vendor_id", "inventory", "category_id", "spec", "address", "username",
            "password", "manage_ip", "gmt_catalog", "task_id", "is_recycle", "gmt_remove", "project_id"};

    // 5.5增加字段:business_id、expired_time、pool_group_id
    // business_id、pool_group_id已和魏伟确认，存null
    // expired_time:已和魏伟确认，需要根据cms的id去object_relation表中根据target和catalog查询
}
