package com.bocloud.sms.service.upgrade.tasks;


import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.util.MultiValueMap;

import javax.sql.DataSource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 资源池数据迁移任务
 *
 * 5.0中的网络资源池会按照平台id绑定到相应的计算资源池上
 *
 * @author luyanchao
 * @date 2021-07-15
 */
@Component
public class PoolGroupTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOnTasks = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context spring上下文
     * @param userTask 因为有creator_id、mender_id、owner_id
     * @param ipPoolTask 因为要处理port_group_relation表依赖ip_pool表
     * @param serverTask 因为要处理port_group_relation表依赖server表
     * @param dataStoreTask 因为要处理port_group_relation表依赖data_store表
     * @param regionTask 因为要处理port_group_relation表依赖region表
     * @param unchangedTask 因为要处理port_group_relation表依赖port_group和vdc和cluster和store_host表
     */
    @Autowired
    public PoolGroupTask(ApplicationContext context, UserTask userTask, CloudVendorTask cloudVendorTask,
                         IpPoolTask ipPoolTask, ServerTask serverTask, DataStoreTask dataStoreTask,
                         RegionTask regionTask, UnchangedTask unchangedTask) {
        this.context = context;
        this.dependOnTasks.add(userTask);
        this.dependOnTasks.add(cloudVendorTask);
        this.dependOnTasks.add(ipPoolTask);
        this.dependOnTasks.add(serverTask);
        this.dependOnTasks.add(dataStoreTask);
        this.dependOnTasks.add(regionTask);
        this.dependOnTasks.add(unchangedTask);
    }

    @Override
    public String getName() {
        return "资源池数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOnTasks;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        String poolColumeNames = UpgradeUtils.getColumnNameSql(Arrays.asList(POOL_GROUP_COLUMNS));
        String poolColumeValues = UpgradeUtils.getColumnValueSql(Arrays.asList(POOL_GROUP_COLUMNS));

        // 查询5.0的资源池数据
        String sqlSelectPool = " select " + poolColumeNames + " from pool_group where 1=1 ";
        List<Map<String, Object>> poolDatas = fromJdbcTemplate.queryForList(sqlSelectPool, new HashMap<>());
        Map<String, List<Map<String, Object>>> v50PoolDatasByCategory = poolDatas.stream()
                .collect(Collectors.groupingBy(map -> map.get("category").toString()));
        // 将5.0的资源池按计算资源池和网络资源池分组
        List<Map<String, Object>> computePoolDatas = v50PoolDatasByCategory.get("COMPUTER");
        List<Map<String, Object>> networkPoolDatas = v50PoolDatasByCategory.get("NETWORK");

        // 查询5.0资源池关联数据
        String sqlSelectReleation = " select pool_group_id, target, category from pool_group_relation where 1=1 ";
        List<Map<String, Object>> relationDatas = fromJdbcTemplate.queryForList(sqlSelectReleation, new HashMap<>());
        // 将5.0资源池关联数据按资源池id进行分组
        Map<Long, List<Map<String, Object>>> poolRalationDatas = relationDatas.stream()
                .collect(Collectors.groupingBy(map -> Long.parseLong(map.get("pool_group_id").toString())));

        int index = 1;
        // 处理计算资源池
        for (Map<String, Object> computePoolData : computePoolDatas) {
            long poolId = Long.parseLong(computePoolData.get("id").toString());
            long vendorId = Long.parseLong(computePoolData.get("vendor_id").toString());

            // 先把5.0的计算资源池写到5.5
            String sqlInsertPool = " insert into pool_group (" + poolColumeNames + ") values (" + poolColumeValues + ")";
            targetJdbcTemplate.update(sqlInsertPool, computePoolData);

            // 处理新增加的字段 vendor_type
            String sqlSelectVendor = "select type from cloud_vendor where id = :id ";
            List<Map<String, Object>> vendorData = targetJdbcTemplate.queryForList(sqlSelectVendor, Collections.singletonMap("id", vendorId));
            String vendorType = vendorData.get(0).get("type").toString();
            String sqlUpdateVendor = "update pool_group set vendor_type = :type where id = :id ";
            Map<String, Object> paramUpdateVendor = new HashMap<>();
            paramUpdateVendor.put("id", poolId);
            paramUpdateVendor.put("type", vendorType);
            targetJdbcTemplate.update(sqlUpdateVendor, paramUpdateVendor);

            // pool_group_relation
            List<Map<String, Object>> thisPoolRelationDatas = poolRalationDatas.get(poolId);
            if (!CollectionUtils.isEmpty(thisPoolRelationDatas)) {
                String category = thisPoolRelationDatas.get(0).get("category").toString();

                // 处理新增加的字段relation_type
                String sqlUpdateRelationType = "update pool_group set relation_type = :type where id = :id ";
                Map<String, Object> paramUpdateRelationType = new HashMap<>();
                paramUpdateRelationType.put("id", poolId);
                paramUpdateRelationType.put("type", category);
                targetJdbcTemplate.update(sqlUpdateRelationType, paramUpdateRelationType);

                // 将5.0的pool_group_relation表数据写入到5.5
                String sqlInsertRelation = "insert into pool_group_relation " +
                        "(pool_group_id, target, category) values (:pool, :target, :category) ";
                // TODO: 2021/7/16 5.5新增加的networks  store_ids enabled还没处理
                Map<String, Object> paramInsertRelation = new HashMap<>();
                paramInsertRelation.put("pool", poolId);
                for (Map<String, Object> thisPoolRelationData : thisPoolRelationDatas) {
                    if ("Cluster".equals(thisPoolRelationData.get("category"))) {
                        long clusterId = Long.parseLong(thisPoolRelationData.get("target").toString());
                        // 插入资源池和集群关联
                        paramInsertRelation.put("target", clusterId);
                        paramInsertRelation.put("category", "Cluster");
                        targetJdbcTemplate.update(sqlInsertRelation, paramInsertRelation);
                        // 关联了集群，需要查询集群下面的主机，主机下面的存储
                        String sqlSelectHost = "select id from server where cluster_id = :clusterId ";
                        List<Map<String, Object>> hostDatas = targetJdbcTemplate
                                .queryForList(sqlSelectHost, Collections.singletonMap("clusterId", clusterId));
                        for (Map<String, Object> hostData : hostDatas) {
                            long hostId = Long.parseLong(hostData.get("id").toString());
                            // 插入资源池和主机关联
                            paramInsertRelation.put("target", hostId);
                            paramInsertRelation.put("category", "Host");
                            targetJdbcTemplate.update(sqlInsertRelation, paramInsertRelation);
                            // 查询主机关联的存储
                            String sqlSelectStoreHost = "select store_id from store_host " +
                                    "where vendor_id = :vendorId and vhost_id = :hostId and category = 'PM' ";
                            Map<String, Object> paramSelectStoreHost = new HashMap<>();
                            paramSelectStoreHost.put("vendorId", vendorId);
                            paramSelectStoreHost.put("hostId", hostId);
                            List<Map<String, Object>> hostStoreDatas = targetJdbcTemplate
                                    .queryForList(sqlSelectStoreHost, paramSelectStoreHost);
                            for (Map<String, Object> hostStoreData : hostStoreDatas) {
                                long storeId = Long.parseLong(hostStoreData.get("store_id").toString());
                                // 插入资源池和存储关联
                                paramInsertRelation.put("target", storeId);
                                paramInsertRelation.put("category", "Storage");
                                targetJdbcTemplate.update(sqlInsertRelation, paramInsertRelation);
                            }
                        }
                    } else if ("Host".equals(thisPoolRelationData.get("category"))) {
                        long hostId = Long.parseLong(thisPoolRelationData.get("target").toString());
                        // 插入资源池和主机关联
                        paramInsertRelation.put("target", hostId);
                        paramInsertRelation.put("category", "Host");
                        targetJdbcTemplate.update(sqlInsertRelation, paramInsertRelation);
                        // 查询主机关联的存储
                        String sqlSelectStoreHost = "select store_id from store_host " +
                                "where vendor_id = :vendorId and vhost_id =: hostId and category = 'PM' ";
                        Map<String, Object> paramSelectStoreHost = new HashMap<>();
                        paramInsertRelation.put("vendorId", vendorId);
                        paramInsertRelation.put("hostId", hostId);
                        List<Map<String, Object>> hostStoreDatas = targetJdbcTemplate
                                .queryForList(sqlSelectStoreHost, paramSelectStoreHost);
                        for (Map<String, Object> hostStoreData : hostStoreDatas) {
                            long storeId = Long.parseLong(hostStoreData.get("store_id").toString());
                            // 插入资源池和存储关联
                            paramInsertRelation.put("target", storeId);
                            paramInsertRelation.put("category", "Storage");
                            targetJdbcTemplate.update(sqlInsertRelation, paramInsertRelation);
                        }
                    } else if ("Region".equals(thisPoolRelationData.get("category"))) {
                        long longRegionId = Long.parseLong(thisPoolRelationData.get("target").toString());
                        // 查询区域id
                        String sqlSelectRegion = "select region_id from region where id = :id";
                        List<Map<String, Object>> regionDatas = targetJdbcTemplate
                                .queryForList(sqlSelectRegion, Collections.singletonMap("id", longRegionId));
                        String stringRegionId = regionDatas.get(0).get("region_id").toString();
                        // 插入资源池和区域关联
                        String sqlInsertRegion = "insert into pool_group_zone " +
                                "(vendor_id, pool_group_id, region_id) values (:vendor, :pool, :region) ";
                        // TODO: 2021/7/16 剩余zone_id和checked未处理
                        Map<String, Object> paramInsertRegion = new HashMap<>();
                        paramInsertRegion.put("pool", poolId);
                        paramInsertRegion.put("vendor", vendorId);
                        paramInsertRegion.put("region", stringRegionId);
                        targetJdbcTemplate.update(sqlInsertRegion, paramInsertRegion);
                    }
                }
            }

            this.percent = Math.floorDiv(index * 100, poolDatas.size());
            index++;
        }
        // 再把5.0的网络资源池写到5.5，按平台id去匹配5.5的计算资源池
        for (Map<String, Object> networkPoolData : networkPoolDatas) {
            // 获取5.0的id, vendorId
            long vendorId = Long.parseLong(networkPoolData.get("vendor_id").toString());
            long networkPoolId = Long.parseLong(networkPoolData.get("id").toString());
            // 按vendorId查询5.0的计算资源池
            List<Map<String, Object>> v50TargetPoolDatas = computePoolDatas.stream()
                    .filter(map -> map.get("vendor_id").equals(vendorId)).collect(Collectors.toList());
            for (Map<String, Object> v50TargetPoolData : v50TargetPoolDatas) {
                long computPoolId = Long.parseLong(v50TargetPoolData.get("id").toString());
                // 按IP池信息尝试到VCenter中对应端口组信息
                List<Map<String, Object>> thisPoolRelatonDatas = poolRalationDatas.get(networkPoolId);
                if (!CollectionUtils.isEmpty(thisPoolRelatonDatas)) {
                    for (Map<String, Object> thisPoolRelatonData : thisPoolRelatonDatas) {
                        long ipPoolId = Long.parseLong(thisPoolRelatonData.get("target").toString());
                        // 查询ip池，获取端口组id
                        String sqlSelectIpPool = "select name, pools, cidr, gateway, dns_names, port_group_id " +
                                "from ip_pool where id = :id and vendor_id = :vendorId ";
                        Map<String, Object> paramSelectIpPool = new HashMap<>();
                        paramSelectIpPool.put("id", ipPoolId);
                        paramSelectIpPool.put("vendorId", vendorId);
                        List<Map<String, Object>> ipPoolDatas = targetJdbcTemplate.queryForList(sqlSelectIpPool, paramSelectIpPool);
                        Map<String, Object> ipPool = ipPoolDatas.get(0);
                        // 将该网络资源池按平台id绑定到对应的计算资源池，并绑定端口组
                        if (ipPool.containsKey("port_group_id")) {
                            Long portGroupId = Long.valueOf(ipPool.get("port_group_id").toString());
                            String sqlInsertRelation = "insert into port_group_relation " +
                                    "(ip_pool_id, pool_group_id, port_group_id) values (:ipPool, :poolGroup, :portGroup)";
                            Map<String, Object> paramInsertRelation = new HashMap<>();
                            paramInsertRelation.put("ipPool", ipPoolId);
                            paramInsertRelation.put("poolGroup", computPoolId);
                            paramInsertRelation.put("portGroup", portGroupId);
                            targetJdbcTemplate.update(sqlInsertRelation, paramInsertRelation);
                        }
                    }
                }
            }

            this.percent = Math.floorDiv(index * 100, poolDatas.size());
            index++;
        }
        this.percent = 100;
    }

    private static final String[] POOL_GROUP_COLUMNS = new String[] {
            "id","name","status","is_deleted","remark","props","gmt_create","gmt_modify","creator_id","mender_id",
            "owner_id","parent_id","category","vendor_id","binding_policy","policy_id","priority_type","region",
            "availablitiy_zone","is_public"
    };
}
