package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 区域数据迁移任务
 *
 * 这个表的表结构没有变化，也没有初始化数据
 * 但是在弄资源池平滑迁移的时候发现：5.5中openstack的region数据相比5.0，在region_id中写入了值
 * 所以需要把这个表的处理单拎出来
 *
 * @author luyanchao
 * @date 2021-07-17
 */
@Component
public class RegionTask implements UpgradeTask{

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     * @param cloudVendorTask 依赖CloudVendorTask，因为需要单独处理openstack平台数据
     */
    public RegionTask(ApplicationContext context, UserTask userTask, CloudVendorTask cloudVendorTask) {
        this.context = context;
        this.dependOn.add(userTask);
        this.dependOn.add(cloudVendorTask);
    }

    @Override
    public String getName() {
        return "区域数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 迁移数据
        List<String> columns = UpgradeUtils.getColumns(this.dataSource, "region");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "region", columns.toArray(new String[0]),
                percent -> this.percent = Math.floorDiv(percent * 80, 100));
        this.percent = 80;
        // 如果是openstack平台，将region_id按照code进行更新
        String sql = "update region set region_id = code " +
                "where vendor_id in (select id from cloud_vendor where type = 'OPENSTACK' )";
        targetJdbcTemplate.update(sql, new HashMap<>());
        this.percent = 100;
    }
}
