package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 角色数据迁移任务
 *
 * 5.5中包含角色1 超级管理员 2 任务审核员 3 资产管理员 4 租户API角色
 * 5.0中包含角色1 超级管理员 2 任务审核员 3 资产管理员
 *
 * 1. 查询5.5中id=4的角色
 * 2. 删掉5.5中id!=1的角色
 * 3. 从5.0迁移id!=1角色到5.5
 * 4. 插入5.5中id=4的角色
 * 5. 处理5.5中角色表的creator_id、mender_id、owner_id
 * 6. role表的tenant_id不需要额外处理，当然tentant表数据迁移的时候id不能变化
 *
 * @author luyanchao
 * @date 2021-06-17
 */
@Component
public class RoleTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public RoleTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "角色数据迁移任务";
    }

    @Override
    public int getPercent() {
        return percent;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 获取role表的所有列名称
        List<String> columeNames = UpgradeUtils.getColumns(this.dataSource, "role");
        String sqlColumns = String.join(",", columeNames);
        String sqlValues = columeNames.stream().map(name -> ":" + name).collect(Collectors.joining(", "));
        this.percent = 5;

        // 查询5.5中租户API角色
        String sqlSelectApi = "select " + sqlColumns + " from role where id = 4 ";
        List<Map<String, Object>> apiRoleDatas = targetJdbcTemplate.queryForList(sqlSelectApi, new HashMap<>());
        this.percent = 10;

        // 删掉5.5中非超级管理员之外的角色
        String sqlDelete = "delete from role where id != 1";
        targetJdbcTemplate.update(sqlDelete, new HashMap<>());
        this.percent = 20;

        // 迁移角色数据(超级管理员除外、租户角色除外)
        String sqlSelect = "select " + sqlColumns + " from role where id != 1 and tenant_id = 0";
        List<Map<String, Object>> fromDbDatas = fromJdbcTemplate.queryForList(sqlSelect, new HashMap<>());
        int index = 1;
        for (Map<String, Object> data : fromDbDatas) {
            String sqlInsert = "insert into role (" + sqlColumns + ") values (" + sqlValues + ") ";
            targetJdbcTemplate.update(sqlInsert, data);
            this.percent = Math.floorDiv(index * 40, fromDbDatas.size()) + 20;
            index++;
        }
        this.percent = 60;

        // 插入5.5中租户API角色，注意不要写入id，否则可能会主键冲突
        for (Map<String, Object> apiRoleData : apiRoleDatas) {
            List<String> apiRoleColumeNames = new ArrayList<>(columeNames);
            apiRoleColumeNames.remove("id");
            String sqlInsert = "insert into role (" + String.join(",", apiRoleColumeNames) + ") " +
                    "values (" + apiRoleColumeNames.stream().map(s -> ":" + s).collect(Collectors.joining(",")) + ") ";
            targetJdbcTemplate.update(sqlInsert, apiRoleData);
        }
        this.percent = 70;

        // 处理creator_id、mender_id、owner_id
        index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "role", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "role", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "role", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 30, idMap.size()) + 70;
            index++;
        }
        this.percent = 100;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }
}
