package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.function.Consumer;

/**
 * 脚本仓库数据迁移任务
 *
 * @author luyanchao
 * @date 2021-07-08
 */
@Component
public class ScriptTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public ScriptTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "脚本仓库数据迁移任务";
    }

    @Override
    public int getPercent() {
        return percent;
    }

    /**
     * 迁移script_group表，并生成5.5新增script_group数据的id变化关系
     *
     * @param callback 进度回调，取值范围0-100
     * @return id变化关系， key 5.5标准库id  value 迁移完自动id
     */
    private Map<Long, Long> group(Consumer<Integer> callback) {
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 获取列名称
        List<String> columns = UpgradeUtils.checkColumns(this.dataSource, this.context.getBean(DataSource.class), "script_group");
        // 5.5标准库相对于5.0标准库新增数据id为9 12 13 14，先查出来
        String sqlSelect55 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script_group where id in (9, 12, 13, 14)";
        List<Map<String, Object>> v55Datas = targetJdbcTemplate.queryForList(sqlSelect55, new HashMap<>());
        // 查询5.0数据。5.0中有一个lmh_test1数据肯定没用，所以这里过滤掉
        String sqlSelect50 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script_group where 1=1 ";
        List<Map<String, Object>> v50Datas = fromJdbcTemplate.queryForList(sqlSelect50, new HashMap<>());
        // 截断5.5数据
        String sqlTruncate = "delete from script_group";
        targetJdbcTemplate.update(sqlTruncate, new HashMap<>());

        int index = 1, total = v55Datas.size() + v50Datas.size();
        // 5.0到5.5的id对应关系
        Map<Long, Long> groupIdMap = new HashMap<>();
        // 导入5.0的数据，包含id，并记录id对应关系
        String sqlInsert50 = "insert into script_group (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v50Data : v50Datas) {
            targetJdbcTemplate.update(sqlInsert50, v50Data);

            long v50Id = Long.parseLong(v50Data.get("id").toString());
            groupIdMap.put(v50Id, v50Id);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }
        // 导入5.5的数据，id自动生成，并记录id对应关系
        columns.remove("id");
        String sqlInsert55 = "insert into script_group (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v55Data : v55Datas) {
            KeyHolder keyHolder = new GeneratedKeyHolder();
            SqlParameterSource parameterSource = new MapSqlParameterSource(v55Data);
            targetJdbcTemplate.update(sqlInsert55, parameterSource, keyHolder);

            long v55OldId = Long.parseLong(v55Data.get("id").toString());
            long v55NewId = Objects.requireNonNull(keyHolder.getKey()).longValue();
            groupIdMap.put(v55OldId, v55NewId);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }
        // 因为id变了，所以还需要处理自身parentId
        String sqlUpdate = "update script_group set parent_id = :pid where id = :id ";
        index = 1;
        for (Map<String, Object> v55Data : v55Datas) {
            long oldPid = Long.parseLong(v55Data.get("parent_id").toString());
            if (oldPid == 0) {
                continue;
            }
            long oldId = Long.parseLong(v55Data.get("id").toString());
            long newPid = groupIdMap.get(oldPid);
            long newId = groupIdMap.get(oldId);

            Map<String, Object> paramUpdate = new HashMap<>();
            paramUpdate.put("id", newId);
            paramUpdate.put("pid", newPid);
            targetJdbcTemplate.update(sqlUpdate, paramUpdate);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, v55Datas.size()) + 50);
            }
            index++;
        }
        // 处理creator_id、mender_id、owner_id
        index = 1;
        Map<Long, Long> userIdMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : userIdMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_group", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_group", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_group", "owner_id", id.getKey(), id.getValue());

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, userIdMap.size()) + 75);
            }
            index++;
        }
        return groupIdMap;
    }

    /**
     * 迁移script表，并生成5.5新增script数据的id变化关系
     * @param groupIdMap 5.5相比于5.0新增script_group数据的id对应情况
     * @param callback 进度回调，取值范围0-100
     * @return id变化关系， key 5.5标准库id  value 迁移完自动id
     */
    private Map<Long, Long> script(Map<Long, Long> groupIdMap, Consumer<Integer> callback) {
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 获取列名称
        List<String> columns = UpgradeUtils.checkColumns(this.dataSource, this.context.getBean(DataSource.class), "script");
        // 5.5标准库相对于5.0标准库新增数据为id从82-96，先查出来
        String sqlSelect55 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script where id between 82 and 96";
        List<Map<String, Object>> v55Datas = targetJdbcTemplate.queryForList(sqlSelect55, new HashMap<>());
        // 查询5.0数据。5.0中有一个lmh_test1数据肯定没用，所以这里过滤掉
        String sqlSelect50 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script where name != 'lmh_test1' ";
        List<Map<String, Object>> v50Datas = fromJdbcTemplate.queryForList(sqlSelect50, new HashMap<>());
        // 截断5.5数据
        String sqlTruncate = "delete from script";
        targetJdbcTemplate.update(sqlTruncate, new HashMap<>());

        int index = 1, total = v55Datas.size() + v50Datas.size();
        // 5.0到5.5的id对应关系
        Map<Long, Long> scriptIdMap = new HashMap<>();
        // 导入5.0的数据，包含id，并记录id对应关系
        String sqlInsert50 = "insert into script (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v50Data : v50Datas) {
            targetJdbcTemplate.update(sqlInsert50, v50Data);

            long v50OldId = Long.parseLong(v50Data.get("id").toString());
            scriptIdMap.put(v50OldId, v50OldId);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }
        // 导入5.5的数据，id自动生成，并记录id对应关系
        columns.remove("id");
        String sqlInsert55 = "insert into script (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v55Data : v55Datas) {
            KeyHolder keyHolder = new GeneratedKeyHolder();
            SqlParameterSource parameterSource = new MapSqlParameterSource(v55Data);
            targetJdbcTemplate.update(sqlInsert55, parameterSource, keyHolder);

            long v55OldId = Long.parseLong(v55Data.get("id").toString());
            long v55NewId = Objects.requireNonNull(keyHolder.getKey()).longValue();
            scriptIdMap.put(v55OldId, v55NewId);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }
        // 由于script_group表的id变化会引起当前表的group_id的变化，所以这里要处理一下
        String sqlUpdate = "update script set group_id = :groupId where id = :id ";
        index = 1;
        for (Map<String, Object> v55Data : v55Datas) {
            long oldId = Long.parseLong(v55Data.get("id").toString());
            long oldGroupId = Long.parseLong(v55Data.get("group_id").toString());
            long newId = scriptIdMap.get(oldId);
            long newGroupId = groupIdMap.get(oldGroupId);
            Map<String, Object> paramUpdate = new HashMap<>();
            paramUpdate.put("id", newId);
            paramUpdate.put("groupId", newGroupId);
            targetJdbcTemplate.update(sqlUpdate, paramUpdate);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, v50Datas.size()) + 50);
            }
            index++;
        }
        // 处理creator_id、mender_id、owner_id
        index = 1;
        Map<Long, Long> userIdMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : userIdMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script", "owner_id", id.getKey(), id.getValue());

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, userIdMap.size()) + 75);
            }
            index++;
        }
        return scriptIdMap;
    }

    /**
     * 迁移script_content表
     *
     * @param scriptIdMap 5.5相比于5.0新增数据的id对应情况
     * @param callback 进度回调，取值范围0-100
     * @return id变化关系， key 5.5标准库id  value 迁移完自动id
     */
    private Map<Long, Long> content(Map<Long, Long> scriptIdMap, Consumer<Integer> callback) {
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 获取列名称
        List<String> columns = UpgradeUtils.checkColumns(this.dataSource, this.context.getBean(DataSource.class), "script_content");
        // 5.5标准库相对于5.0标准库新增数据为id从82-96，先查出来
        String sqlSelect55 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script_content where script_id between 82 and 96";
        List<Map<String, Object>> v55Datas = targetJdbcTemplate.queryForList(sqlSelect55, new HashMap<>());
        // 查询5.0数据。5.0中有一个lmh_test1数据肯定没用，所以这里过滤掉
        String sqlSelect50 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script_content " +
                "where script_id not in (select id from script where name = 'lmh_test1') ";
        List<Map<String, Object>> v50Datas = fromJdbcTemplate.queryForList(sqlSelect50, new HashMap<>());
        // 截断5.5数据
        String sqlTruncate = "delete from script_content";
        targetJdbcTemplate.update(sqlTruncate, new HashMap<>());

        int index = 1, total = v55Datas.size() + v50Datas.size();
        // 5.0到5.5的id对应关系，并记录id对应关系
        Map<Long, Long> contentIdMap = new HashMap<>();
        // 导入5.0的数据，包含id
        String sqlInsert50 = "insert into script_content (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v50Data : v50Datas) {
            targetJdbcTemplate.update(sqlInsert50, v50Data);

            long v50OldId = Long.parseLong(v50Data.get("id").toString());
            contentIdMap.put(v50OldId, v50OldId);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }
        // 导入5.5的数据，id自动生成，并记录id对应关系
        columns.remove("id");
        String sqlInsert55 = "insert into script_content (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v55Data : v55Datas) {
            KeyHolder keyHolder = new GeneratedKeyHolder();
            SqlParameterSource parameterSource = new MapSqlParameterSource(v55Data);
            targetJdbcTemplate.update(sqlInsert55, parameterSource, keyHolder);

            long v55OldId = Long.parseLong(v55Data.get("id").toString());
            long v55NewId = Objects.requireNonNull(keyHolder.getKey()).longValue();
            contentIdMap.put(v55OldId, v55NewId);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }
        // 处理script表id变化引起的script_content的script_id变化
        String sqlUpdate = "update script_content set script_id = :scriptId where id = :id ";
        index = 1;
        for (Map<String, Object> v55Data : v55Datas) {
            long oldId = Long.parseLong(v55Data.get("id").toString());
            long oldScriptId = Long.parseLong(v55Data.get("script_id").toString());
            long newId = contentIdMap.get(oldId);
            long newScriptId = scriptIdMap.get(oldScriptId);

            Map<String, Object> paramUpdate = new HashMap<>();
            paramUpdate.put("id", newId);
            paramUpdate.put("scriptId", newScriptId);
            targetJdbcTemplate.update(sqlUpdate, paramUpdate);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, v50Datas.size()) + 50);
            }
            index++;
        }
        // 处理creator_id、mender_id、owner_id
        index = 1;
        Map<Long, Long> userIdMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : userIdMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_content", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_content", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_content", "owner_id", id.getKey(), id.getValue());
            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, userIdMap.size()) + 75);
            }
            index++;
        }
        return contentIdMap;
    }

    /**
     * 迁移script_version表
     *
     * @param scriptIdMap 5.5相比于5.0新增script数据的id对应情况
     * @param contentIdMap 5.5相比于5.0新增script_content数据的id对应情况
     * @param callback 进度回调，取值范围0-100
     */
    private void version(Map<Long, Long> scriptIdMap, Map<Long, Long> contentIdMap, Consumer<Integer> callback) {
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 获取列名称
        List<String> columns = UpgradeUtils.checkColumns(this.dataSource, this.context.getBean(DataSource.class), "script_version");
        // 5.5标准库相对于5.0标准库新增数据为id从82-96，先查出来
        String sqlSelect55 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script_version where script_id between 82 and 96";
        List<Map<String, Object>> v55Datas = targetJdbcTemplate.queryForList(sqlSelect55, new HashMap<>());
        // 查询5.0数据。5.0中有一个lmh_test1数据肯定没用，所以这里过滤掉
        String sqlSelect50 = "select " + UpgradeUtils.getColumnNameSql(columns) + " from script_version " +
                "where script_id not in (select id from script where name = 'lmh_test1') ";
        List<Map<String, Object>> v50Datas = fromJdbcTemplate.queryForList(sqlSelect50, new HashMap<>());
        // 截断5.5数据
        String sqlTruncate = "delete from script_version";
        targetJdbcTemplate.update(sqlTruncate, new HashMap<>());

        int index = 1, total = v55Datas.size() + v50Datas.size();
        // 导入5.0的数据，包含id
        String sqlInsert50 = "insert into script_version (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v50Data : v50Datas) {
            targetJdbcTemplate.update(sqlInsert50, v50Data);
            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }
        // 导入5.5的数据，id自动生成，并记录id对应关系
        Map<Long, Long> versionIdMap = new HashMap<>();
        columns.remove("id");
        String sqlInsert55 = "insert into script_version (" + UpgradeUtils.getColumnNameSql(columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(columns) + ") ";
        for (Map<String, Object> v55Data : v55Datas) {
            KeyHolder keyHolder = new GeneratedKeyHolder();
            SqlParameterSource parameterSource = new MapSqlParameterSource(v55Data);
            targetJdbcTemplate.update(sqlInsert55, parameterSource, keyHolder);

            long standardId = Long.parseLong(v55Data.get("id").toString());
            long newId = Objects.requireNonNull(keyHolder.getKey()).longValue();
            versionIdMap.put(standardId, newId);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 50, total));
            }
            index++;
        }

        // 处理script表id变化引起的script_version的script_id变化
        // 处理script_content表id变化引起的script_version的content_id变化
        String sqlUpdate = "update script_version set script_id = :scriptId, content_id = :contentId where id = :id ";
        index = 1;
        for (Map<String, Object> v55Data : v55Datas) {
            long oldId = Long.parseLong(v55Data.get("id").toString());
            long oldScriptId = Long.parseLong(v55Data.get("script_id").toString());
            long oldContentId = Long.parseLong(v55Data.get("content_id").toString());
            long newId = versionIdMap.get(oldId);
            long newScriptId = scriptIdMap.get(oldScriptId);
            long newContentId = contentIdMap.get(oldContentId);

            Map<String, Object> paramUpdate = new HashMap<>();
            paramUpdate.put("id", newId);
            paramUpdate.put("scriptId", newScriptId);
            paramUpdate.put("contentId", newContentId);
            targetJdbcTemplate.update(sqlUpdate, paramUpdate);

            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, v50Datas.size()) + 50);
            }
            index++;
        }

        // 处理creator_id、mender_id、owner_id
        index = 1;
        Map<Long, Long> userIdMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : userIdMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_version", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_version", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "script_version", "owner_id", id.getKey(), id.getValue());
            if (callback != null) {
                callback.accept(Math.floorDiv(index * 25, userIdMap.size()) + 75);
            }
            index++;
        }
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 处理表script_group
        Map<Long, Long> groupIdMap = this.group(percent -> this.percent = Math.floorDiv(percent, 4));
        this.percent = 25;
        // 处理表script
        Map<Long, Long> scriptIdMap = this.script(groupIdMap, percent -> this.percent = Math.floorDiv(percent, 4) + 25);
        this.percent = 50;
        // 处理表script_content
        Map<Long, Long> contentIdMap = this.content(scriptIdMap, percent -> this.percent = Math.floorDiv(percent, 4) + 50);
        this.percent = 75;
        // 处理表script_version
        this.version(scriptIdMap, contentIdMap, percent -> this.percent = Math.floorDiv(percent, 4) + 75);
        this.percent = 100;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }
}
