package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * service_catalog数据迁移任务
 *
 * 5.0和5.5比较
 *
 * 1. 5.5新加了字段has_category
 * 2. 5.5的字段props中多了值
 * 3. 5.5删掉了id=10的数据
 *
 * 所以迁移的时候5.5中存在的code不迁移，维持5.5的样子。其他数据直接迁移
 *
 * @author luyanchao
 * @date 2021-07-02
 */
@Component
public class ServiceCatalogTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     * @param serviceCategoryTask 依赖ServiceCategoryTask，因为有has_category
     */
    public ServiceCatalogTask(ApplicationContext context, UserTask userTask, ServiceCategoryTask serviceCategoryTask) {
        this.context = context;
        this.dependOn.add(userTask);
        this.dependOn.add(serviceCategoryTask);
    }

    @Override
    public String getName() {
        return "service_catalog数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 迁移数据
        List<String> v50Columns = UpgradeUtils.getColumns(this.dataSource, "service_catalog");
        String sqlInsert = "insert into service_catalog (" + UpgradeUtils.getColumnNameSql(v50Columns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(v50Columns) + ") ";
        String sqlSelect = "select " + UpgradeUtils.getColumnNameSql(v50Columns) + " from service_catalog where id > 10";
        List<Map<String, Object>> v50Datas = fromJdbcTemplate.queryForList(sqlSelect, new HashMap<>());
        int index = 1;
        for (Map<String, Object> v50Data : v50Datas) {
            targetJdbcTemplate.update(sqlInsert, v50Data);
            this.percent = Math.floorDiv(index * 90, v50Datas.size());
            index++;
        }
        this.percent = 90;
        // 处理has_category
        String sqlUpdate = "update service_catalog aa left join " +
                "  (select b.catalog_id, count(distinct a.id) as cnt " +
                "   from service_category a join cloud_service b on a.service_id = b.id " +
                "   group by b.catalog_id) bb on aa.id = bb.catalog_id " +
                "set aa.has_category = case when ifnull(bb.cnt, 0) > 0 then 1 else 0 end " +
                "where 1=1";
        targetJdbcTemplate.update(sqlUpdate, new HashMap<>());
        this.percent = 100;
    }

}
