package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * service_category数据迁移任务
 *
 * 由于
 * 1. 5.5相对于5.0的cloud_service初始化数据有增减
 * 2. 相同code的cloud_service在5.0和5.5中的id也不一样
 * 3. 迁移cloud_service的时候直接放弃了5.0的数据，维持5.5的数据不变
 * 所以
 * 1. 查询5.5的cloud_service的code
 * 2. 按照第一步中查询的code到5.0中查询service_category
 * 3. 迁移第二步的数据到5.5，并注意5.5中已有的code不迁移、还需要处理id冲突的问题
 *
 * @author luyanchao
 * @date 2021-07-02
 */
@Component
public class ServiceCategoryTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     * @param cloudServiceTask 依赖CloudServiceTask，因为cloud_service的id有变化
     */
    public ServiceCategoryTask(ApplicationContext context, UserTask userTask, CloudServiceTask cloudServiceTask) {
        this.context = context;
        this.dependOn.add(userTask);
        this.dependOn.add(cloudServiceTask);
    }

    @Override
    public String getName() {
        return "service_category数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 查询5.5中cloud_service数据
        String sqlSelect55Service = "select id, code from cloud_service where 1=1";
        List<Map<String, Object>> services = targetJdbcTemplate.queryForList(sqlSelect55Service, new HashMap<>());
        // 获取service_category的所有列名称
        List<String> categoryColumns = UpgradeUtils.getColumns(this.dataSource, "service_category");

        // 准备迁移过程中要用到的sql
        // 生成查询service_category的sql脚本，查询的时候需要查询出来id
        String sqlSelectCategory = "select " + UpgradeUtils.getColumnNameSql(categoryColumns) +
                " from service_category where service_id = :id ";
        // 生成插入service_category的sql脚本，插入的时候不写入id字段，可能会发生id冲突
        categoryColumns.remove("id");
        String sqlInsert = "insert into service_category (" + UpgradeUtils.getColumnNameSql(categoryColumns) + ") " +
                "values (" + UpgradeUtils.getColumnValueSql(categoryColumns) + ") ";
        String sqlSelect50Service = "select id from cloud_service where code = :code";
        // 迁移service_category数据
        int index = 1;
        for (Map<String, Object> v55service : services) {
            String serviceCode = v55service.get("code").toString();
            long service55Id = Long.parseLong(v55service.get("id").toString());
            List<Map<String, Object>> v50Service = fromJdbcTemplate
                    .queryForList(sqlSelect50Service, Collections.singletonMap("code", serviceCode));
            // 5.0中就没这个cloud_service就不用管了
            if (v50Service.isEmpty()) {
                continue;
            }
            long service50Id = Long.parseLong(v50Service.get(0).get("id").toString());
            List<Map<String, Object>> v50Categories = fromJdbcTemplate
                    .queryForList(sqlSelectCategory, Collections.singletonMap("id", service50Id));
            List<Map<String, Object>> v55Categories = fromJdbcTemplate
                    .queryForList(sqlSelectCategory, Collections.singletonMap("id", service55Id));
            List<String> v55CategoryCodes = v55Categories.stream().map(category -> category.get("code"))
                    .filter(Objects::nonNull).map(Object::toString).collect(Collectors.toList());
            for (Map<String, Object> v50Category : v50Categories) {
                long v50CategoryId = Long.parseLong(v50Category.get("id").toString());
                String v50CategoryCode = v50Category.get("code").toString();
                // 5.5中已经有这个category也不用管了
                if (v55CategoryCodes.contains(v50CategoryCode)) {
                    continue;
                }
                // 插入category数据
                Map<String, Object> paramInsert = new HashMap<>(v50Category);
                paramInsert.put("service_id", service55Id);
                targetJdbcTemplate.update(sqlInsert, paramInsert);
            }
            // 计算百分比
            this.percent = Math.floorDiv(index * 100, services.size());
            index++;
        }
        this.percent = 100;
    }
}
