package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 用户角色关联数据迁移任务
 *
 * 1. 迁移用户数据的时候，没动admin；迁移角色数据的时候，也没动超级管理员，所以迁移用户角色关联的时候也不动
 * 2. 从5.0迁移manager_role的数据到5.5，user_role的数据直接丢弃
 * 3. 处理manager_role的userId
 *
 * @author luyanchao
 * @date 2021-06-17
 */
@Component
public class UserRoleTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    @Autowired
    public UserRoleTask(ApplicationContext context, UserTask userTask, RoleTask roleTask) {
        this.context = context;
        this.dependOn.add(userTask);
        this.dependOn.add(roleTask);
    }

    @Override
    public String getName() {
        return "用户角色关联数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 删掉5.5中非admin<-->超级管理员的关联数据
        String sqlDeleteManager = "delete from manager_role where manager_id != 1 and role_id != 1";
        targetJdbcTemplate.update(sqlDeleteManager, new HashMap<>());
        this.percent = 20;

        // 迁移manager_role
        int index = 1;
        String sqlSelectManager = "select manager_id, role_id from manager_role where manager_id != 1 and role_id != 1 ";
        String sqlInsertManager = "insert into manager_role (manager_id, role_id) values (:manager_id, :role_id) ";
        List<Map<String, Object>> managerDatas = fromJdbcTemplate.queryForList(sqlSelectManager, new HashMap<>());
        for (Map<String, Object> managerData : managerDatas) {
            targetJdbcTemplate.update(sqlInsertManager, managerData);
            this.percent = Math.floorDiv(index * 40, managerDatas.size()) + 20;
        }
        this.percent = 60;

        // 由于5.0的user和manager表合并到5.5的user表会引起id的变化，所以还要处理manager_role.manager_id
        index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "manager_role", "manager_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 40, idMap.size()) + 60;
        }
        this.percent = 100;
    }
}
