package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 用户数据迁移任务
 *
 * 1. 用户数据迁移任务不要依赖于其他任务，有太多的表有creator_id、mender_id、owner_id这三个字段需要等着该任务好了才能进行
 * 2. user表的tenant_id和depart_id不需要额外处理，当然tentant表和department表数据迁移的时候id不能变化
 * 3. 5.0为manager表和user表，5.5合并到user表
 * 4. 5.0中manager表和user表可能会出现重复的id和account，5.5不会
 * 5. 舍弃5.0中的id，5.5中id重新排序
 * 6. 不会删除5.5原本的admin账户，从5.0迁移数据到5.5的时候也不会迁移admin账户
 * @author luyanchao
 * @date 2021-06-17
 */
@Component
public class UserTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private DataSource dataSource;

    /**
     * 构造方法
     * 用户数据迁移任务不要依赖于其他任务
     * @param context spring上下文
     */
    @Autowired
    public UserTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public String getName() {
        return "用户数据迁移任务";
    }

    @Override
    public int getPercent() {
        return percent;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        List<String> userColumns = new ArrayList<>(Arrays.asList(COLUMNS));
        List<String> managerColumns = userColumns.stream().filter(column -> !"tenant_id".equals(column)).collect(Collectors.toList());

        // 迁移用户数据（超级管理员除外）
        String managerSelectSql = "select " + UpgradeUtils.getColumnNameSql(managerColumns) + " from manager where id != 1";
        List<Map<String, Object>> managers = fromJdbcTemplate.queryForList(managerSelectSql, new HashMap<>());
        String userSelectSql = "select " + UpgradeUtils.getColumnNameSql(userColumns) + " from user where 1=1 ";
        List<Map<String, Object>> users = fromJdbcTemplate.queryForList(userSelectSql, new HashMap<>());
        int index = 1, total = managers.size() + users.size();
        for (Map<String, Object> manager : managers) {
            String mamagerInsertSql = "insert into user (is_manager, " + UpgradeUtils.getColumnNameSql(managerColumns) + ") " +
                    "values (1, "+ UpgradeUtils.getColumnValueSql(managerColumns) +")";
            targetJdbcTemplate.update(mamagerInsertSql, manager);
            this.percent = Math.floorDiv(index * 50, total);
            index++;
        }
        for (Map<String, Object> user : users) {
            String userInsertSql = "insert into user (is_manager, " + UpgradeUtils.getColumnNameSql(userColumns) + ") " +
                    "values (0, "+ UpgradeUtils.getColumnValueSql(userColumns) +")";
            targetJdbcTemplate.update(userInsertSql, user);
            this.percent = Math.floorDiv(index * 50, total);
            index++;
        }
        this.percent = 50;

        // 获取5.0到5.5的用户id对应关系
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));

        // 处理user表的creator_id、mender_id、owner_id
        index = 1;
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "user", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "user", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "user", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    private static final String[] COLUMNS = new String[]{
            "name", "account", "password", "remark", "email", "phone", "mobile", "gmt_create", "gmt_modify",
            "creator_id", "mender_id", "status", "is_deleted", "owner_id", "props", "company", "session_id",
            "login_status", "depart_id", "depart_ids", "job_number", "sex", "tenant_id", "portrait",
            "origin", "last_pwd_modify_date", "last_login_date"
    };


}
