package com.bocloud.sms.service.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.Encryptor;
import com.bocloud.common.encrypt.SHAEncryptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.sms.entity.AccountSecurity;
import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.repository.AccountSecurityRepository;
import com.bocloud.sms.repository.DepartmentRepository;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.service.ldap.LdapConnector;
import com.bocloud.sms.service.ldap.LdapRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

public class LdapTransportor {

    private static final Logger logger = LoggerFactory.getLogger(LdapTransportor.class);
    private final UserRepository userRepository;
    private final AccountSecurityRepository accountSecurityRepository;
    private final DepartmentRepository departmentRepository;
    private LdapConnector ldapConnector;
    private LdapRepository ldapRepository;

    private LdapConfig ldapConfig;


    public LdapTransportor(UserRepository userRepository, DepartmentRepository departmentDao, AccountSecurityRepository accountSecurityRepository, LdapConfig config) {
        super();
        this.userRepository = userRepository;
        this.departmentRepository = departmentDao;
        this.accountSecurityRepository = accountSecurityRepository;
        this.ldapConfig = config;
        init(config);
    }

    public boolean accessable() {
        return ldapConnector.accessable();
    }

    private void init(LdapConfig config) {
        this.ldapConnector = new LdapConnector(config.getUrl(), config.getAccount(), config.getPassword(),
                config.getBaseDn());
        this.ldapRepository = new LdapRepository(ldapConnector);
        ldapConnector.start();
    }

    /**
     * 同步
     *
     * @return
     */
    public BocloudResult sync() {
        try {
            if (!ldapConnector.accessable()) {
                return new BocloudResult(false, "Ldap平台认证失败");
            }
            JSONObject object = ldapRepository.listAll(ldapConfig.getLoginDn());
            List<Department> depts = JSONArray.parseArray(object.getString("depts"), Department.class);
            logger.info("获取组织机构信息:" + object.getString("depts"));
            logger.info("缓存中的组织信息:" + JSONObject.toJSONString(depts));
            logger.info("获取LDAP用户信息:" + object.getString("managers"));
            logger.info("start sync ldap data to db……");
            List<Department> deptLists = listChild(0L);
//            logger.info("old:"+ JSON.toJSONString(deptLists));
//            logger.info("new:"+ JSON.toJSONString(depts));
//            logger.info("depts:"+depts.size());
            syncDept(deptLists, depts);
            logger.info("end sync ldap data to db……");
        } catch (Exception e) {
            logger.info("sync ldap info error:", e);
        } finally {
            if (null != ldapConnector) {
                ldapConnector.close();
            }
        }
        return new BocloudResult(true, "success");
    }

    /**
     * 同步组织机构
     *
     * @throws Exception
     */
    private void syncDept(List<Department> oldDepts, List<Department> newDepts) throws Exception {
//        logger.info("start");
        Map<String, User> accountMapUser = userRepository.list().stream().parallel().collect(Collectors.toMap(User::getAccount, Function.identity(), (k1, k2) -> k1));
        if (ListTool.isEmpty(newDepts)) {
            return;
        }
        Map<Long, Department> idMapDept = new HashMap<>(16, 0.75F);
//        logger.info("ListTool.isEmpty(oldDepts):"+ListTool.isEmpty(oldDepts));
        if (ListTool.isEmpty(oldDepts)) {
            syncAllToDb(newDepts, accountMapUser, 0L, idMapDept);
        }else {
            Map<String, Department> ouMapNewDept = deptMap(newDepts, "");
//            logger.info("ouMapNewDept:"+JSON.toJSONString(ouMapNewDept));
            syncDept(oldDepts, accountMapUser, ouMapNewDept, "", idMapDept);
            syncDeptToDb(newDepts, accountMapUser, ouMapNewDept, "", 0L, idMapDept);
        }
    }

    /**
     * 同步所有数据到数据库
     *
     * @param departments
     * @param parentId
     * @param idMap
     * @throws Exception
     */
    private void syncAllToDb(List<Department> departments, Map<String, User> dbUserMap, Long parentId, Map<Long, Department> idMap)
            throws Exception {
        if (ListTool.isEmpty(departments)) {
            return;
        }
        for (Department department : departments) {
            List<Department> children = department.getChildren();
            List<User> managers = department.getUsers();
            department.setCreatorId(1L);
            department.setMenderId(1L);
            department.setTenantId(0L);
            department.setGmtCreate(new Date());
            department.setGmtModify(department.getGmtCreate());
            department.setStatus("NORMAL");
            department.setParentId(parentId);
            department.setOrigin("LDAP");
            departmentRepository.save(department);
            idMap.put(department.getId(), department);
            String ids = "[" + getDeptIds(department.getId() + "", department.getParentId(), idMap) + "]";
            saveUsers(managers, dbUserMap, department.getId(), ids, true);
            syncAllToDb(children, dbUserMap,department.getId(), idMap);
        }
    }

    private String getDeptIds(String ids, Long parentId, Map<Long, Department> idMap) {
        Department parent = idMap.get(parentId);
        if (null == parent) {
            return ids;
        }
        if (parent.getParentId() == null || parent.getParentId() == 0) {
            return parentId + "," + ids;
        }
        ids = parentId + "," + ids;
        return getDeptIds(ids, parent.getParentId(), idMap);
    }

    private void saveUsers(List<User> managers, Map<String, User> dbUserMap,Long deptId, String deptIds, boolean hasDept) throws Exception {
        if (ListTool.isEmpty(managers)) {
            return;
        }
        List<List<User>> addUsers = new ArrayList<>();
        List<User> userList = new ArrayList<>();
        Map<String, String> saltMap = new HashMap<>(16, 0.75F);
        Encryptor encryptor = new SHAEncryptor();
        for (User manager : managers) {
            manager.setMenderId(1L);
            manager.setGmtModify(manager.getGmtCreate());
            if (dbUserMap.containsKey(manager.getAccount())) {
                User user = dbUserMap.get(manager.getAccount());
                user.setEmail(manager.getEmail());
                user.setName(manager.getName());
                user.setMobile(manager.getMobile());
                user.setDepartId(deptId);
                user.setDepartIds(deptIds);
                userRepository.update(user);
                continue;
            }
            manager.setCreatorId(1L);
            manager.setGmtCreate(new Date());
            manager.setStatus("NORMAL");
            manager.setDepartId(deptId);
            manager.setDepartIds(deptIds);
            manager.setOrigin("LDAP");
            manager.setIsManager(true);
            manager.setDeleted(false);
            String salt = UUID.randomUUID().toString();
            String password = !StringUtils.hasText(manager.getPassword()) ? "123456" : manager.getPassword();
            manager.setPassword(encryptor.encrypt(password, salt));
            userList.add(manager);
            saltMap.put(manager.getAccount(), salt);
            if (userList.size() == 1000) {
                addUsers.add(userList);
                userList = new ArrayList<>();
            }
        }
        if (!userList.isEmpty()) {
            addUsers.add(userList);
        }
        for (List<User> list : addUsers) {
            if (!list.isEmpty()) {
                userRepository.queryProxy().batch(list).batchSize(1000).doInsert();
            }
        }
        List<User> allUsers = this.userRepository.listByOrigin(deptId, "LDAP", hasDept);
        for (User manager : allUsers) {
            if (!StringUtils.hasText(manager.getAccount()) || !saltMap.containsKey(manager.getAccount())) {
                continue;
            }
            AccountSecurity security = new AccountSecurity();
            String data = Long.toString(System.currentTimeMillis());
            security.setTargetId(manager.getId());
            security.setCategory(AccountSecurity.Category.User);
            security.setTenantId(0L);
            security.setSalt(saltMap.get(manager.getAccount()));
            security.setApiKey(encryptor.encrypt(data, Common.API_KEY));
            security.setSecKey(encryptor.encrypt(data, Common.SEC_KEY));
            accountSecurityRepository.save(security);
        }
    }

    /**
     * 更新数据库已有的组织机构数据
     *
     * @param oldDepts
     * @param ouMapNewDept
     * @param name
     * @throws Exception
     */
    private void syncDept(List<Department> oldDepts, Map<String, User> accountMapUser, Map<String, Department> ouMapNewDept, String name,
                          Map<Long, Department> idMap) throws Exception {
        for (Department oldDept : oldDepts) {
            idMap.put(oldDept.getId(), oldDept);
            String dn = "ou=" + oldDept.getName() + (!StringUtils.hasText(name) ? "" : "," + name);
            logger.info("dn:"+dn);
            if (ouMapNewDept.containsKey(dn)) {//数据库已存在的部门
                Department newDept = ouMapNewDept.get(dn);
                //设置新部门的ID为旧部门的id
                newDept.setId(oldDept.getId());
                //根据部门关系，拼接部门ID，用于用户关联
                String ids = "[" + getDeptIds(oldDept.getId() + "", oldDept.getParentId(), idMap) + "]";
//                logger.info("users:"+JSON.toJSONString(newDept.getUsers()));
//                logger.info("oldDept:"+JSON.toJSONString(oldDept));
                syncUser(oldDept.getUsers(), accountMapUser, newDept.getUsers(), oldDept.getId(), ids, true);
                ouMapNewDept.remove(dn);
                if (!ListTool.isEmpty(oldDept.getChildren())) {
                    syncDept(oldDept.getChildren(), accountMapUser, ouMapNewDept, dn, idMap);
                }
            }
        }
    }

    /**
     * 同步用户
     *
     * @param dbUsers
     * @param ldapUsers
     * @param deptId
     * @throws Exception
     */
    private void syncUser(List<User> dbUsers, Map<String, User> dbUserMap, List<User> ldapUsers, Long deptId, String deptIds, boolean hasDept)
            throws Exception {
        if (ListTool.isEmpty(ldapUsers)) {
            return;
        }
        //如果数据中不存在用户，则保存全部的ldap的用户
        if (ListTool.isEmpty(dbUsers)) {
            saveUsers(ldapUsers, dbUserMap, deptId, deptIds, hasDept);
            return;
        }
        Map<String, User> accountMapLdapUser = new HashMap<>(16, 0.75F);
        for (User manager : ldapUsers) {
            accountMapLdapUser.put(manager.getAccount(), manager);
        }
        Encryptor encryptor = new SHAEncryptor();
        for (User manager : dbUsers) {
            //ldap中包含数据库中的用户
            if (accountMapLdapUser.containsKey(manager.getAccount())) {
                User ldapUser = accountMapLdapUser.get(manager.getAccount());
                //更新用户数据
                if (!StringUtils.hasText(manager.getAccount()) || !manager.getAccount().equals(ldapUser.getAccount())) {
                    manager.setAccount(ldapUser.getAccount());
                    manager.setMenderId(1L);
                    manager.setGmtModify(new Date());
                    manager.setDepartId(deptId);
                    manager.setDepartIds(deptIds);
                    manager.setEmail(ldapUser.getEmail());
                    userRepository.update(manager);
                }
                //从ldap的用户map中删除
                accountMapLdapUser.remove(manager.getAccount());
            }
        }
        //剩下的这些是数据库中没有的用户
        for (String key : accountMapLdapUser.keySet()) {
            User manager = accountMapLdapUser.get(key);
            if (dbUserMap.containsKey(manager.getAccount())) {//这个分支应该不会进入
                User user = dbUserMap.get(manager.getAccount());
//                logger.info("flag:"+true);
                user.setMenderId(1L);
                user.setGmtModify(new Date());
                user.setEmail(manager.getEmail());
                user.setDepartId(deptId);
                user.setDepartIds(deptIds);
                user.setMobile(manager.getMobile());
//                logger.info("manager:"+JSON.toJSONString(user));
                userRepository.update(user);
                continue;
            }
            manager.setCreatorId(1L);
            manager.setMenderId(1L);
            manager.setGmtCreate(new Date());
            manager.setGmtModify(manager.getGmtCreate());
            manager.setStatus("NORMAL");
            manager.setDepartId(deptId);
            manager.setOrigin("LDAP");
            manager.setIsManager(true);
            String salt = UUID.randomUUID().toString();
            String password = !StringUtils.hasText(manager.getPassword()) ? "123456" : manager.getPassword();
            manager.setPassword(encryptor.encrypt(password, salt));
            userRepository.save(manager);
            AccountSecurity security = new AccountSecurity();
            String data = Long.toString(System.currentTimeMillis());
            security.setTargetId(manager.getId());
            security.setCategory(AccountSecurity.Category.User);
            security.setTenantId(0L);
            security.setSalt(salt);
            security.setApiKey(encryptor.encrypt(data, Common.API_KEY));
            security.setSecKey(encryptor.encrypt(data, Common.SEC_KEY));
            accountSecurityRepository.save(security);
        }
    }

    /**
     * 同步组织机构到数据库
     *
     * @param departments
     * @param ouMapNewDept
     * @param name
     * @param parentId
     * @throws Exception
     */
    private void syncDeptToDb(List<Department> departments, Map<String, User> dbUserMap, Map<String, Department> ouMapNewDept, String name,
                              Long parentId, Map<Long, Department> idMap) throws Exception {
        if (ListTool.isEmpty(departments)) {
            return;
        }
        for (Department department : departments) {
            String dn = "ou=" + department.getName() + (!StringUtils.hasText(name) ? "" : "," + name);
            logger.info("保存部门：{}", dn);
            //遍历所有的ldap部门，ouMapNewDept代表不存在于数据库中的部门，如果不存在数据库则进行插入和保存，并且更新用户
            if (ouMapNewDept.containsKey(dn)) {
                department.setCreatorId(1L);
                department.setMenderId(1L);
                department.setTenantId(0L);
                department.setGmtCreate(new Date());
                department.setGmtModify(department.getGmtCreate());
                department.setStatus("NORMAL");
                department.setParentId(parentId);
                department.setOrigin("LDAP");
                departmentRepository.save(department);
                idMap.put(department.getId(), department);
                String ids = "[" + getDeptIds(department.getId() + "", department.getParentId(), idMap) + "]";
                if("基础设施部".equals(department.getName())){
                    logger.info("id:"+department.getId());
                    logger.info("user:"+JSON.toJSON(department.getUsers()));
                }
                //更新用户
                List<User> managers = department.getUsers();
                saveUsers(managers, dbUserMap, department.getId(), ids, true);
            }
            syncDeptToDb(department.getChildren(), dbUserMap, ouMapNewDept, dn, department.getId(), idMap);
        }
    }

    private List<Department> listChild(Long parentId) throws Exception {
        List<Department> departs = departmentRepository.list(parentId, 0L, "LDAP");
        if (!ListTool.isEmpty(departs)) {
            for (Department depart : departs) {
                depart.setChildren(listChild(depart.getId()));
                depart.setUsers(userRepository.listByOrigin(depart.getId(), "LDAP", true));
            }
        }
        return departs;
    }

    private Map<String, Department> deptMap(List<Department> departs, String name) {
//        logger.info("departs:"+departs.size());
//        logger.info("departs:"+JSON.toJSONString(departs));
        if (ListTool.isEmpty(departs)) {
            return new HashMap<>(1);
        }
        Map<String, Department> deptMap = new HashMap<>(16, 0.75F);
        for (Department depart : departs) {
//            logger.info("depart:"+JSON.toJSONString(depart));
//            logger.info("depart.name"+depart.getName());
            String dn = "ou=" + depart.getName() + (!StringUtils.hasText(name) ? "" : "," + name);
            deptMap.put(dn, depart);
            deptMap.putAll(deptMap(depart.getChildren(), dn));
        }
        return deptMap;
    }
}
