package com.bocloud.sms.repository;

import com.bocloud.common.enums.BaseStatus;
import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.orm.OrmGenericDaoImpl;
import com.bocloud.sms.entity.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/2/3 周一 11:37
 */
@Repository
public class UserRepository extends OrmGenericDaoImpl<User, Long> {
    @Autowired
    public UserRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 删除用户
     *
     * @param id
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean delete(Long id, Long menderId) throws BoCloudDBException {
        String sql = "update users set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 锁定用户
     *
     * @param id
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean lock(Long id, Long menderId) throws BoCloudDBException {
        return updateStatus(id, menderId, BaseStatus.ABNORMAL);
    }

    /**
     * 批量冻结用户
     *
     * @param users
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean batchLock(List<User> users, Long menderId) throws BoCloudDBException {
        for (User user : users) {
            user.setStatus(BaseStatus.ABNORMAL.name());
            user.setMenderId(menderId);
        }
        return batchUpdate(users, users.size());
    }

    public boolean batchUpdateStatus(List<Long> ids, Long menderId, BaseStatus status) {
        String sql = "update users set status = :status, gmt_modify = :gmtModify, mender_id = :menderId  where id in (:ids) and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("status", status.name());
        return this.execute(sql, params) > 0;
    }

    /**
     * 批量解冻用户
     *
     * @param users
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean batchActive(List<User> users, Long menderId) throws BoCloudDBException {
        for (User user : users) {
            user.setStatus(BaseStatus.NORMAL.name());
            user.setMenderId(menderId);
        }
        return batchUpdate(users, users.size());
    }


    /**
     * 激活用户
     *
     * @param id
     * @param menderId
     * @return
     * @throws BoCloudDBException
     */
    public boolean active(Long id, Long menderId) throws BoCloudDBException {
        return updateStatus(id, menderId, BaseStatus.NORMAL);
    }

    private boolean updateStatus(Long id, Long menderId, BaseStatus status) {
        String sql = "update users set status = :status, gmt_modify = :gmtModify, mender_id = :menderId  where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("status", status.name());
        return this.execute(sql, params) > 0;
    }

    /**
     * 根据账号查询用户是否存在
     *
     * @param account
     * @return
     * @throws BoCloudDBException
     */
    public User getByAccount(String account, Long tenantId) throws BoCloudDBException {
        String sql = "select u.*, t.account tenant_account, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.account = :account and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("account", account);
        params.put("tenantId", tenantId);
        List<User> list = this.list(User.class, sql, params);
        Assert.isTrue(list.size() <= 1, "该账号已存在");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据账号查询用户
     *
     * @param account
     * @return
     * @throws BoCloudDBException
     */
    public User getByAccount(String account) throws BoCloudDBException {
        String sql = "select u.*, t.account tenant_account,t.name tenant_name, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.account = :account ";
        Map<String, Object> params = MapTools.simpleMap("account", account);
        List<User> list = this.list(User.class, sql, params);
        Assert.isTrue(list.size() <= 1, "该账号已存在");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 分页查询用户
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     * @throws BoCloudDBException
     */
    public List<User> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long roleId) throws BoCloudDBException {
        String sql;
        if (null != roleId && 0L != roleId) {
            sql = "select a.*,d.name depart_name from users a left join manager_role b on a.id = b.manager_id" +
                    " left join department d on d.id = a.depart_id where a.id != 1 and a.is_deleted = 0 and b.role_id = " + roleId;
        } else {
            sql = "select a.*,b.name tenant_name,b.id tenant_id from users a left join tenant b on b.id = a.tenant_id where a.is_deleted = 0 and a.id != 1";
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        // 20220319 这里如果roleId是null会导致bre报错，而且这行毛细用不到吧，直接干掉
        // paramMap.put("roleId", roleId);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 简易查询用户
     *
     * @param params
     * @param sorter
     * @return
     * @throws BoCloudDBException
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, Long roleId) throws BoCloudDBException {
        String sql;
        if (null != roleId && 0L != roleId) {
            sql = "select  a.* from users a left join manager_role b on a.id = b.manager_id  where  a.is_deleted = 0 and a.id != 1 and b.role_id = " + roleId;
        } else {
            sql = "select a.* from users a where a.is_deleted = 0 and a.id != 1 ";
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(User.class, sql, paramMap).stream().map(object -> new SimpleBean(object.getId(), object.getName())).collect(Collectors.toList());
    }

    public List<User> list() throws BoCloudDBException {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.id != 1";
        return this.list(User.class, sql);
    }

    /**
     * 查询用户数量
     *
     * @param params
     * @return
     * @throws BoCloudDBException
     */
    public int count(List<Param> params, Long roleId) throws BoCloudDBException {
        String sql;
        if (null == roleId || 0L == roleId) {
            sql = "select count(a.id) from users a where a.is_deleted = 0 and a.id != 1";
        } else {
            sql = "select count(a.id) from users a left join manager_role b on a.id = b.manager_id where a.id != 1 and a.is_deleted = 0 and b.role_id = " + roleId;
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("roleId", roleId);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 查询用户数量
     *
     * @param params     查询参数
     * @param projectIds 项目id
     * @param roleId     角色id
     * @return
     * @throws BoCloudDBException
     */
    public int count(List<Param> params, List<Long> projectIds, Long roleId) throws BoCloudDBException {
        String sql;
        if (null != roleId) {
            if (projectIds.size() > 0) {
                sql = "select count(distinct a.id) from users a left join user_role b on a.id = b.user_id left join user_project c on a.id = c.user_id where a.is_deleted = 0 "
                        + " and b.role_id = " + roleId + "and c.project_id in (:projectIds) ";
            } else {
                sql = "select count(1) from users a left join user_role b on a.id = b.user_id where a.is_deleted = 0 and b.role_id = "
                        + roleId;
            }
        } else {
            if (projectIds.size() > 0) {
                sql = "select count(distinct a.id) from users a left join user_project b on a.id = b.user_id where a.is_deleted = 0 and b.project_id in (:projectIds) ";
            } else {
                sql = "select count(1) from users a where a.is_deleted = 0 ";
            }
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (projectIds.size() > 0) {
            paramMap.put("projectIds", projectIds);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 根据id查询用户
     *
     * @param id
     * @return
     * @throws BoCloudDBException
     */
    public User query(Long id) throws BoCloudDBException {
        String sql = "select u.*, t.account tenant_account, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<User> list = this.list(User.class, sql, paramMap);
        Assert.isTrue(list.size() <= 1, "more than one users in table");
        return list.isEmpty() ? null : list.get(0);
    }

    public List<User> listByIds(List<Long> ids) throws BoCloudDBException {
        String sql = "select u.*, t.account tenant_account, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.id in (:ids) ";
        Map<String, Object> paramMap = MapTools.simpleMap("ids", ids);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 根据邮箱查询用户
     *
     * @param email
     * @return
     * @throws BoCloudDBException
     */
    public User getByEmail(String email) throws BoCloudDBException {
        String sql = "select * from users where email = :email and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.EMAIL, email);
        List<User> list = this.list(User.class, sql, params);
        Assert.isTrue(list.size() <= 1, "该邮箱已被注册");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据租户ID查询用户
     *
     * @param tenantId
     * @return
     * @throws BoCloudDBException
     */
    public List<User> listByTenant(Long tenantId) throws BoCloudDBException {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.tenant_id = :tenantId";
        Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 根据权限查询用户
     *
     * @param router
     * @return
     * @throws BoCloudDBException
     */
    public List<User> listByPermission(String router) throws BoCloudDBException {
        String sql = "select distinct u.* from users u left join user_role ur on u.id = ur.user_id left join role r on r.id = ur.role_id and r.is_deleted = 0 "
                + "left join role_permission rp on r.id = rp.role_id left join permission p on p.id = rp.permission_id where u.is_deleted = 0  and p.router =:router";
        Map<String, Object> paramMap = MapTools.simpleMap("router", router);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 查询租户下用户
     *
     * @param projectId
     * @return
     * @throws BoCloudDBException
     */
    public List<User> listProjectUser(Long projectId) throws BoCloudDBException {
        String sql = "select distinct a.* from users a left join user_project on a.id = user_project.user_id where project_id = :projectId ";
        Map<String, Object> paramMap = MapTools.simpleMap("projectId", projectId);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 查询项目负责人
     *
     * @param projectId
     * @return
     * @throws BoCloudDBException
     */
    public List<User> getProjectManagers(Long projectId) throws BoCloudDBException {
        String sql = "select * from users u left join  user_project up on u.id = up.user_id where up.project_id = :projectId and up.project_admin = 1";
        Map<String, Object> params = MapTools.simpleMap("projectId", projectId);
        return this.list(User.class, sql, params);
    }

    public int projectUserCount(List<Param> params) throws BoCloudDBException {
        String sql = "select count(distinct b.id) from users b left join user_project up on b.id = up.user_id where b.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "up");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<User> listProjectUser(Integer page, Integer rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select distinct a.*,b.name tenant_name,b.id tenant_id ,up.project_admin project_manager from users a left join tenant b on b.id = a.tenant_id left join user_project up on a.id = up.user_id where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "up");
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, null, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(User.class, sql, paramMap);
    }

    public List<User> listManagerHasExpiredDate() {
        String sql = "select * from users where is_deleted = 0 and expired_date is not null";
        return this.list(User.class, sql);
    }


    /**
     * 根据组织机构ID查询
     *
     * @param departId
     * @return
     * @throws BoCloudDBException
     */
    public List<User> listByDepartment(Long departId) throws BoCloudDBException {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.depart_id = :departId";
        Map<String, Object> paramMap = MapTools.simpleMap("departId", departId);
        return this.list(User.class, sql, paramMap);
    }

    public List<User> listByOrigin(Long departId, String origin, boolean hasDept) throws BoCloudDBException {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.origin = :origin";
        Map<String, Object> paramMap = MapTools.simpleMap("origin", origin);
        if (hasDept) {
            if (null != departId) {
                sql += " and a.depart_id = :departId";
                paramMap.put("departId", departId);
            }
        } else {
            sql += " and a.depart_id is null";
        }
        return this.list(User.class, sql, paramMap);
    }

    /** @Author: lxy
     * @Description: 根据租户id修改租户下所有用户组织
     * @Date 2021/6/15 14:36
     * @Param id
     * @param departId
     * @param departIds
     * @return boolean
     * @throws:
     **/
    public boolean updateDepartIdByTenantId(Long id, Long departId, String departIds) {
        String sql = "update users set depart_id = :departId , depart_ids = :departIds where tenant_id=:id";
        Map<String, Object> paramsMap = MapTools.simpleMap("id", id);
        paramsMap.put("departId", departId);
        paramsMap.put("departIds", departIds);
        return super.execute(sql,paramsMap) > 0;
    }

    public List<User> listByProjectId(Long projectId) {
        String sql = "select * from users u left join user_project p on u.id = p.user_id where p.project_id = :projectId";
        Map<String, Object> paramMap = MapTools.simpleMap("projectId", projectId);
        return this.list(User.class, sql, paramMap);

    }
}
